#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.

"""
Implementation of a DirectoryEditor demo plugin for Traits UI demo program.

This demo shows each of the four styles of the DirectoryEditor
"""

# Imports:
from __future__ import absolute_import
from traits.api \
    import HasTraits, Directory

from traitsui.api \
    import Item, Group, View

# Define the demo class:


class DirectoryEditorDemo(HasTraits):
    """ Define the main DirectoryEditor demo class. """

    # Define a Directory trait to view:
    dir_name = Directory

    # Display specification (one Item per editor style):
    dir_group = Group(
        Item('dir_name', style='simple', label='Simple'),
        Item('_'),
        Item('dir_name', style='custom', label='Custom'),
        Item('_'),
        Item('dir_name', style='text', label='Text'),
        Item('_'),
        Item('dir_name', style='readonly', label='ReadOnly')
    )

    # Demo view:
    view = View(
        dir_group,
        title='DirectoryEditor',
        buttons=['OK'],
        resizable=True
    )

# Create the demo:
demo = DirectoryEditorDemo()

# Run the demo (if invoked from the command line):
if __name__ == '__main__':
    demo.configure_traits()



"""
Monitoring threads in the user interface

Shows a simple user interface being updated by multiple threads.

When the *Start Threads* button is pressed, the program starts three independent
threads running. Each thread counts from 0 to 199, updating its own
thread-specific trait, and performs a sleep of a thread-specific duration
between each update.

The *Start Threads* button is disabled while the threads are running, and
becomes active again once all threads have finished running.
"""

from __future__ import absolute_import
from threading import Thread
from time import sleep
from traits.api import HasTraits, Int, Button
from traitsui.api import View, Item, VGroup


class ThreadDemo(HasTraits):

    # The thread specific counters:
    thread_0 = Int
    thread_1 = Int
    thread_2 = Int

    # The button used to start the threads running:
    start = Button('Start Threads')

    # The count of how many threads ae currently running:
    running = Int

    view = View(
        VGroup(
            Item('thread_0', style='readonly'),
            Item('thread_1', style='readonly'),
            Item('thread_2', style='readonly'),
        ),
        '_',
        Item('start', show_label=False,
             enabled_when='running == 0'),
        resizable=True,
        width=250,
        title='Monitoring threads'
    )

    def _start_changed(self):
        for i in range(3):
            Thread(target=self.counter,
                   args=('thread_%d' % i, (i * 10 + 10) / 1000.0)).start()

    def counter(self, name, interval):
        self.running += 1
        count = 0
        for i in range(200):
            setattr(self, name, count)
            count += 1
            sleep(interval)
        self.running -= 1

# Create the demo:
demo = ThreadDemo()

# Run the demo (if invoked from the command line):
if __name__ == '__main__':
    demo.configure_traits()



"""
Implementation of a FontEditor demo plugin for Traits UI demo program.

This demo shows each of the four styles of the FontEditor.
"""

from __future__ import absolute_import
from traits.api import HasTraits, Font
from traitsui.api import Item, Group, View

#-------------------------------------------------------------------------
#  Demo Class
#-------------------------------------------------------------------------


class FontEditorDemo(HasTraits):
    """ This class specifies the details of the FontEditor demo.
    """

    # To demonstrate any given Trait editor, an appropriate Trait is required.
    font_trait = Font

    # Display specification (one Item per editor style)
    font_group = Group(Item('font_trait', style='simple', label='Simple'),
                       Item('_'),
                       Item('font_trait', style='custom', label='Custom'),
                       Item('_'),
                       Item('font_trait', style='text', label='Text'),
                       Item('_'),
                       Item('font_trait',
                            style='readonly',
                            label='ReadOnly'))

    # Demo view
    view1 = View(font_group,
                 title='FontEditor',
                 buttons=['OK'])


# Create the demo:
popup = FontEditorDemo()

# Run the demo (if invoked from the command line):
if __name__ == '__main__':
    popup.configure_traits()



"""This demo shows how to use Traits TreeEditors with PyTables to walk the
heirarchy of an HDF5 file.  This only picks out arrays and groups, but could
easily be extended to other structures, like tables.

In the demo, the path to the selected item is printed whenever the selection
changes.  In order to run, a path to an existing HDF5 database must be given
at the bottom of this file.
"""
from __future__ import absolute_import, print_function

from traits.api import HasTraits, Str, List, Instance
from traitsui.api import TreeEditor, TreeNode, View, Item, Group

import tables as tb


# View for objects that aren't edited
no_view = View()


# HDF5 Nodes in the tree
class Hdf5ArrayNode(HasTraits):
    name = Str('<unknown>')
    path = Str('<unknown>')
    parent_path = Str('<unknown>')


class Hdf5GroupNode(HasTraits):
    name = Str('<unknown>')
    path = Str('<unknown>')
    parent_path = Str('<unknown>')
    # Can't have recursive traits?  Really?
    #groups = List( Hdf5GroupNode )
    groups = List
    arrays = List(Hdf5ArrayNode)
    groups_and_arrays = List


class Hdf5FileNode(HasTraits):
    name = Str('<unknown>')
    path = Str('/')
    groups = List(Hdf5GroupNode)
    arrays = List(Hdf5ArrayNode)
    groups_and_arrays = List

# Recurssively build tree, there is probably a better way of doing this.


def _get_sub_arrays(group, h5file):
    """Return a list of all arrays immediately below a group in an HDF5 file."""
    l = []

    for array in h5file.iter_nodes(group, classname='Array'):
        a = Hdf5ArrayNode(
            name=array._v_name,
            path=array._v_pathname,
            parent_path=array._v_parent._v_pathname,
        )
        l.append(a)

    return l


def _get_sub_groups(group, h5file):
    """Return a list of all groups and arrays immediately below a group in an HDF5 file."""
    l = []

    for subgroup in h5file.iter_nodes(group, classname='Group'):
        g = Hdf5GroupNode(
            name=subgroup._v_name,
            path=subgroup._v_pathname,
            parent_path=subgroup._v_parent._v_pathname,
        )

        subarrays = _get_sub_arrays(subgroup, h5file)
        if subarrays != []:
            g.arrays = subarrays

        subgroups = _get_sub_groups(subgroup, h5file)
        if subgroups != []:
            g.groups = subgroups

        g.groups_and_arrays = []
        g.groups_and_arrays.extend(subgroups)
        g.groups_and_arrays.extend(subarrays)

        l.append(g)

    return l


def _hdf5_tree(filename):
    """Return a list of all groups and arrays below the root group of an HDF5 file."""

    h5file = tb.open_file(filename, 'r')

    file_tree = Hdf5FileNode(
        name=filename,
        groups=_get_sub_groups(h5file.root, h5file),
        arrays=_get_sub_arrays(h5file.root, h5file),
    )

    file_tree.groups_and_arrays = []
    file_tree.groups_and_arrays.extend(file_tree.groups)
    file_tree.groups_and_arrays.extend(file_tree.arrays)

    h5file.close()

    return file_tree

# Get a tree editor


def _hdf5_tree_editor(selected=''):
    """Return a TreeEditor specifically for HDF5 file trees."""
    return TreeEditor(
        nodes=[
            TreeNode(
                node_for=[Hdf5FileNode],
                auto_open=True,
                children='groups_and_arrays',
                label='name',
                view=no_view,
            ),
            TreeNode(
                node_for=[Hdf5GroupNode],
                auto_open=False,
                children='groups_and_arrays',
                label='name',
                view=no_view,
            ),
            TreeNode(
                node_for=[Hdf5ArrayNode],
                auto_open=False,
                children='',
                label='name',
                view=no_view,
            ),
        ],
        editable=False,
        selected=selected,
    )


if __name__ == '__main__':
    import sys
    from traits.api import Any

    class ATree(HasTraits):
        h5_tree = Instance(Hdf5FileNode)
        node = Any

        traits_view = View(
            Group(
                Item('h5_tree',
                     editor=_hdf5_tree_editor(selected='node'),
                     resizable=True,
                     show_label=False,
                     ),
                orientation='vertical',
            ),
            title='HDF5 Tree Example',
            buttons=['Undo', 'OK', 'Cancel'],
            resizable=True,
            width=.3,
            height=.3
        )

        def _node_changed(self):
            print(self.node.path)

    a_tree = ATree(h5_tree=_hdf5_tree(sys.argv[1]))
    a_tree.configure_traits()
#    a_tree.edit_traits()



"""
Demonstrate the CSVListEditor class.<br>
<br>
This editor allows the user to enter a *single* line of input text, containing
comma-separated values (or another separator may be specified). Your program
specifies an element Trait type of Int, Float, Str, Enum, or Range.
"""
from __future__ import absolute_import, print_function

from traits.api import HasTraits, List, Int, Float, Enum, Range, Str, Button, \
    Property
from traitsui.api import View, Item, Label, Heading, VGroup, HGroup, UItem, \
    spring, TextEditor, CSVListEditor


class Demo(HasTraits):

    list1 = List(Int)

    list2 = List(Float)

    list3 = List(Str, maxlen=3)

    list4 = List(Enum('red', 'green', 'blue', 2, 3))

    list5 = List(Range(low=0.0, high=10.0))

    # 'low' and 'high' are used to demonstrate lists containing dynamic ranges.
    low = Float(0.0)
    high = Float(1.0)

    list6 = List(Range(low=-1.0, high='high'))

    list7 = List(Range(low='low', high='high'))

    pop1 = Button("Pop from first list")

    sort1 = Button("Sort first list")

    # This will be str(self.list1).
    list1str = Property(Str, depends_on='list1')

    traits_view = \
        View(
            HGroup(
                # This VGroup forms the column of CSVListEditor examples.
                VGroup(
                    Item('list1', label="List(Int)",
                         editor=CSVListEditor(ignore_trailing_sep=False),
                         tooltip='options: ignore_trailing_sep=False'),
                    Item('list1', label="List(Int)", style='readonly',
                         editor=CSVListEditor()),
                    Item('list2', label="List(Float)",
                         editor=CSVListEditor(enter_set=True, auto_set=False),
                         tooltip='options: enter_set=True, auto_set=False'),
                    Item('list3', label="List(Str, maxlen=3)",
                         editor=CSVListEditor()),
                    Item('list4',
                         label="List(Enum('red', 'green', 'blue', 2, 3))",
                         editor=CSVListEditor(sep=None),
                         tooltip='options: sep=None'),
                    Item('list5', label="List(Range(low=0.0, high=10.0))",
                         editor=CSVListEditor()),
                    Item('list6', label="List(Range(low=-1.0, high='high'))",
                         editor=CSVListEditor()),
                    Item('list7', label="List(Range(low='low', high='high'))",
                         editor=CSVListEditor()),
                    springy=True,
                ),
                # This VGroup forms the right column; it will display the
                # Python str representation of the lists.
                VGroup(
                    UItem('list1str', editor=TextEditor(),
                          enabled_when='False', width=240),
                    UItem('list1str', editor=TextEditor(),
                          enabled_when='False', width=240),
                    UItem('list2', editor=TextEditor(),
                          enabled_when='False', width=240),
                    UItem('list3', editor=TextEditor(),
                          enabled_when='False', width=240),
                    UItem('list4', editor=TextEditor(),
                          enabled_when='False', width=240),
                    UItem('list5', editor=TextEditor(),
                          enabled_when='False', width=240),
                    UItem('list6', editor=TextEditor(),
                          enabled_when='False', width=240),
                    UItem('list7', editor=TextEditor(),
                          enabled_when='False', width=240),
                ),
            ),
            '_',
            HGroup('low', 'high', spring, UItem('pop1'), UItem('sort1')),
            Heading("Notes"),
            Label("Hover over a list to see which editor options are set, "
                  "if any."),
            Label("The editor of the first list, List(Int), uses "
                  "ignore_trailing_sep=False, so a trailing comma is "
                  "an error."),
            Label("The second list is a read-only view of the first list."),
            Label("The editor of the List(Float) example has enter_set=True "
                  "and auto_set=False; press Enter to validate."),
            Label("The List(Str) example will accept at most 3 elements."),
            Label("The editor of the List(Enum(...)) example uses sep=None, "
                  "i.e. whitespace acts as a separator."),
            Label("The last two List(Range(...)) examples take one or both "
                  "of their limits from the Low and High fields below."),
            width=720,
            title="CSVListEditor Demonstration",
        )

    def _list1_default(self):
        return [1, 4, 0, 10]

    def _get_list1str(self):
        return str(self.list1)

    def _pop1_fired(self):
        if len(self.list1) > 0:
            x = self.list1.pop()
            print(x)

    def _sort1_fired(self):
        self.list1.sort()


if __name__ == "__main__":
    demo = Demo()
    demo.configure_traits()



from __future__ import absolute_import
from unittest import TestCase

from traits.api import HasTraits, Str, Int, RGBColor, List
from traitsui.api import View, Group, Item, TableEditor, ObjectColumn
from traitsui.color_column import ColorColumn

from traitsui.tests._tools import skip_if_null, store_exceptions_on_all_threads

class MyEntry(HasTraits):
    name= Str()
    value=Int(0)
    color=RGBColor()

    entry_view = View(
        Group(
            Item('name'),
            Item('value'),
            Item('color')
        )
    )

my_editor = TableEditor(
    columns=[
        ObjectColumn(name='name'),
        ObjectColumn(name='value'),
        ColorColumn(name='color', style='readonly')
    ],
    orientation='vertical',
    show_toolbar=True,
    row_factory=MyEntry,
)

class MyData(HasTraits):
    data_list = List(MyEntry)

    view=View(
        Item(
            'data_list',
            editor=my_editor,
            show_label=False
        ),

    )


class TestColorColumn(TestCase):

    @skip_if_null
    def test_color_column(self):
        # Behaviour: column ui should display without error

        with store_exceptions_on_all_threads():
            d1 = MyEntry(name='a', value=2, color=(1.0, 0.3, 0.1))
            d2 = MyEntry(name='b', value=3, color=(0.1, 0.0, 0.9))
            data = MyData(data_list=[d1, d2])

            ui = data.edit_traits()

            ui.dispose()



"""
Implementation of a ColorEditor demo plugin for Traits UI demo program.

This demo shows each of the four styles of the ColorEditor.
"""

from __future__ import absolute_import
from traits.api import HasTraits, Color
from traitsui.api import Item, Group, View

#-------------------------------------------------------------------------
#  Demo Class
#-------------------------------------------------------------------------


class ColorEditorDemo(HasTraits):
    """ This class specifies the details of the ColorEditor demo.
    """

    # To demonstrate any given Trait editor, an appropriate Trait is required.
    color_trait = Color

    # Items are used to define the demo display - one item per
    # editor style
    color_group = Group(
        Item(
            'color_trait', style='simple', label='Simple'), Item('_'), Item(
            'color_trait', style='custom', label='Custom'), Item('_'), Item(
                'color_trait', style='text', label='Text'), Item('_'), Item(
                    'color_trait', style='readonly', label='ReadOnly'))

    # Demo view
    view1 = View(color_group,
                 title='ColorEditor',
                 buttons=['OK'])


# Create the demo:
popup = ColorEditorDemo()

# Run the demo (if invoked from the command line):
if __name__ == '__main__':
    popup.configure_traits()



from __future__ import absolute_import

from pyface.gui import GUI
from traits.api import HasTraits, Instance, Int, List, Str, Tuple

from traitsui.api import (
    EvalTableFilter, Item, ObjectColumn, TableEditor, View
)
from traitsui.tests._tools import (
    is_current_backend_qt4, is_current_backend_wx, press_ok_button,
    skip_if_not_qt4,  skip_if_null, store_exceptions_on_all_threads
)


class ListItem(HasTraits):
    """ Items to visualize in a table editor """
    value = Str
    other_value = Int


class ObjectListWithSelection(HasTraits):
    values = List(Instance(ListItem))
    selected = Instance(ListItem)
    selections = List(Instance(ListItem))
    selected_index = Int
    selected_indices = List(Int)
    selected_column = Str
    selected_columns = List(Str)
    selected_cell = Tuple(Instance(ListItem), Str)
    selected_cells = List(Tuple(Instance(ListItem), Str))
    selected_cell_index = Tuple(Int, Int)
    selected_cell_indices = List(Tuple(Int, Int))


class ObjectList(HasTraits):
    values = List(Instance(ListItem))


simple_view = View(
    Item(
        'values',
        show_label=False,
        editor=TableEditor(
            columns=[
                ObjectColumn(name='value'),
                ObjectColumn(name='other_value'),
            ],
        )
    ),
    buttons=['OK'],
)

filtered_view = View(
    Item(
        'values',
        show_label=False,
        editor=TableEditor(
            columns=[
                ObjectColumn(name='value'),
                ObjectColumn(name='other_value'),
            ],
            filter=EvalTableFilter(expression='other_value >= 2'),
        )
    ),
    buttons=['OK'],
)

select_row_view = View(
    Item(
        'values',
        show_label=False,
        editor=TableEditor(
            columns=[
                ObjectColumn(name='value'),
                ObjectColumn(name='other_value'),
            ],
            selection_mode='row',
            selected='selected',
        )
    ),
    buttons=['OK'],
)

select_rows_view = View(
    Item(
        'values',
        show_label=False,
        editor=TableEditor(
            columns=[
                ObjectColumn(name='value'),
            ],
            selection_mode='rows',
            selected='selections',
        )
    ),
    buttons=['OK'],
)

select_row_index_view = View(
    Item(
        'values',
        show_label=False,
        editor=TableEditor(
            columns=[
                ObjectColumn(name='value'),
                ObjectColumn(name='other_value'),
            ],
            selection_mode='row',
            selected_indices='selected_index',
        )
    ),
    buttons=['OK'],
)

select_row_indices_view = View(
    Item(
        'values',
        show_label=False,
        editor=TableEditor(
            columns=[
                ObjectColumn(name='value'),
                ObjectColumn(name='other_value'),
            ],
            selection_mode='rows',
            selected_indices='selected_indices',
        )
    ),
    buttons=['OK'],
)

select_column_view = View(
    Item(
        'values',
        show_label=False,
        editor=TableEditor(
            columns=[
                ObjectColumn(name='value'),
                ObjectColumn(name='other_value'),
            ],
            selection_mode='column',
            selected='selected_column',
        )
    ),
    buttons=['OK'],
)

select_columns_view = View(
    Item(
        'values',
        show_label=False,
        editor=TableEditor(
            columns=[
                ObjectColumn(name='value'),
                ObjectColumn(name='other_value'),
            ],
            selection_mode='columns',
            selected='selected_columns',
        )
    ),
    buttons=['OK'],
)

select_column_index_view = View(
    Item(
        'values',
        show_label=False,
        editor=TableEditor(
            columns=[
                ObjectColumn(name='value'),
                ObjectColumn(name='other_value'),
            ],
            selection_mode='column',
            selected_indices='selected_index',
        )
    ),
    buttons=['OK'],
)

select_column_indices_view = View(
    Item(
        'values',
        show_label=False,
        editor=TableEditor(
            columns=[
                ObjectColumn(name='value'),
                ObjectColumn(name='other_value'),
            ],
            selection_mode='columns',
            selected_indices='selected_indices',
        )
    ),
    buttons=['OK'],
)

select_cell_view = View(
    Item(
        'values',
        show_label=False,
        editor=TableEditor(
            columns=[
                ObjectColumn(name='value'),
                ObjectColumn(name='other_value'),
            ],
            selection_mode='cell',
            selected='selected_cell',
        )
    ),
    buttons=['OK'],
)

select_cells_view = View(
    Item(
        'values',
        show_label=False,
        editor=TableEditor(
            columns=[
                ObjectColumn(name='value'),
                ObjectColumn(name='other_value'),
            ],
            selection_mode='cells',
            selected='selected_cells',
        )
    ),
    buttons=['OK'],
)

select_cell_index_view = View(
    Item(
        'values',
        show_label=False,
        editor=TableEditor(
            columns=[
                ObjectColumn(name='value'),
                ObjectColumn(name='other_value'),
            ],
            selection_mode='cell',
            selected_indices='selected_cell_index',
        )
    ),
    buttons=['OK'],
)

select_cell_indices_view = View(
    Item(
        'values',
        show_label=False,
        editor=TableEditor(
            columns=[
                ObjectColumn(name='value'),
                ObjectColumn(name='other_value'),
            ],
            selection_mode='cells',
            selected_indices='selected_cell_indices',
        )
    ),
    buttons=['OK'],
)


@skip_if_null
def test_table_editor():
    gui = GUI()
    object_list = ObjectListWithSelection(
        values=[ListItem(value=str(i**2)) for i in range(10)]
    )

    with store_exceptions_on_all_threads():
        ui = object_list.edit_traits(view=simple_view)
        gui.process_events()
        press_ok_button(ui)
        gui.process_events()


@skip_if_null
def test_filtered_table_editor():
    gui = GUI()
    object_list = ObjectListWithSelection(
        values=[ListItem(value=str(i**2)) for i in range(10)]
    )

    with store_exceptions_on_all_threads():
        ui = object_list.edit_traits(view=filtered_view)
        gui.process_events()

        filter = ui.get_editors('values')[0].filter

        press_ok_button(ui)
        gui.process_events()

    assert filter is not None


@skip_if_null
def test_table_editor_select_row():
    gui = GUI()
    object_list = ObjectListWithSelection(
        values=[ListItem(value=str(i**2)) for i in range(10)]
    )
    object_list.selected = object_list.values[5]

    with store_exceptions_on_all_threads():
        ui = object_list.edit_traits(view=select_row_view)
        editor = ui.get_editors('values')[0]
        gui.process_events()
        if is_current_backend_qt4():
            selected = editor.selected
        elif is_current_backend_wx():
            selected = editor.selected_row

        press_ok_button(ui)
        gui.process_events()

    assert selected is object_list.values[5]


@skip_if_null
def test_table_editor_select_rows():
    gui = GUI()
    object_list = ObjectListWithSelection(
        values=[ListItem(value=str(i**2)) for i in range(10)]
    )
    object_list.selections = object_list.values[5:7]

    with store_exceptions_on_all_threads():
        ui = object_list.edit_traits(view=select_rows_view)
        editor = ui.get_editors('values')[0]
        gui.process_events()
        if is_current_backend_qt4():
            selected = editor.selected
        elif is_current_backend_wx():
            selected = editor.selected_rows

        press_ok_button(ui)
        gui.process_events()

    assert selected == object_list.values[5:7]


@skip_if_null
def test_table_editor_select_row_index():
    gui = GUI()
    object_list = ObjectListWithSelection(
        values=[ListItem(value=str(i**2)) for i in range(10)]
    )
    object_list.selected_index = 5

    with store_exceptions_on_all_threads():
        ui = object_list.edit_traits(view=select_row_index_view)
        editor = ui.get_editors('values')[0]
        gui.process_events()
        if is_current_backend_qt4():
            selected = editor.selected_indices
        elif is_current_backend_wx():
            selected = editor.selected_row_index

        press_ok_button(ui)
        gui.process_events()

    assert selected == 5


@skip_if_null
def test_table_editor_select_row_indices():
    gui = GUI()
    object_list = ObjectListWithSelection(
        values=[ListItem(value=str(i**2)) for i in range(10)]
    )
    object_list.selected_indices = [5, 7, 8]

    with store_exceptions_on_all_threads():
        ui = object_list.edit_traits(view=select_row_indices_view)
        editor = ui.get_editors('values')[0]
        gui.process_events()
        if is_current_backend_qt4():
            selected = editor.selected_indices
        elif is_current_backend_wx():
            selected = editor.selected_row_indices

        press_ok_button(ui)
        gui.process_events()

    assert selected == [5, 7, 8]


@skip_if_null
def test_table_editor_select_column():
    gui = GUI()
    object_list = ObjectListWithSelection(
        values=[ListItem(value=str(i**2)) for i in range(10)]
    )
    object_list.selected_column = 'value'

    with store_exceptions_on_all_threads():
        ui = object_list.edit_traits(view=select_column_view)
        editor = ui.get_editors('values')[0]
        gui.process_events()
        if is_current_backend_qt4():
            selected = editor.selected
        elif is_current_backend_wx():
            selected = editor.selected_column

        press_ok_button(ui)
        gui.process_events()

    assert selected == 'value'


@skip_if_null
def test_table_editor_select_columns():
    gui = GUI()
    object_list = ObjectListWithSelection(
        values=[ListItem(value=str(i**2)) for i in range(10)]
    )
    object_list.selected_columns = ['value', 'other_value']

    with store_exceptions_on_all_threads():
        ui = object_list.edit_traits(view=select_columns_view)
        editor = ui.get_editors('values')[0]
        gui.process_events()
        if is_current_backend_qt4():
            selected = editor.selected
        elif is_current_backend_wx():
            selected = editor.selected_columns

        press_ok_button(ui)
        gui.process_events()

    assert selected == ['value', 'other_value']


@skip_if_null
def test_table_editor_select_column_index():
    gui = GUI()
    object_list = ObjectListWithSelection(
        values=[ListItem(value=str(i**2)) for i in range(10)]
    )
    object_list.selected_index = 1

    with store_exceptions_on_all_threads():
        ui = object_list.edit_traits(view=select_column_index_view)
        editor = ui.get_editors('values')[0]
        gui.process_events()
        if is_current_backend_qt4():
            selected = editor.selected_indices
        elif is_current_backend_wx():
            selected = editor.selected_column_index

        press_ok_button(ui)
        gui.process_events()

    assert selected == 1


@skip_if_null
def test_table_editor_select_column_indices():
    gui = GUI()
    object_list = ObjectListWithSelection(
        values=[ListItem(value=str(i**2)) for i in range(10)]
    )
    object_list.selected_indices = [0, 1]

    with store_exceptions_on_all_threads():
        ui = object_list.edit_traits(view=select_column_indices_view)
        editor = ui.get_editors('values')[0]
        gui.process_events()
        if is_current_backend_qt4():
            selected = editor.selected_indices
        elif is_current_backend_wx():
            selected = editor.selected_column_indices

        press_ok_button(ui)
        gui.process_events()

    assert selected == [0, 1]


@skip_if_null
def test_table_editor_select_cell():
    gui = GUI()
    object_list = ObjectListWithSelection(
        values=[ListItem(value=str(i**2)) for i in range(10)]
    )
    object_list.selected_cell = (object_list.values[5], 'value')

    with store_exceptions_on_all_threads():
        ui = object_list.edit_traits(view=select_cell_view)
        editor = ui.get_editors('values')[0]
        gui.process_events()
        if is_current_backend_qt4():
            selected = editor.selected
        elif is_current_backend_wx():
            selected = editor.selected_cell

        press_ok_button(ui)
        gui.process_events()

    assert selected == (object_list.values[5], 'value')


@skip_if_null
def test_table_editor_select_cells():
    gui = GUI()
    object_list = ObjectListWithSelection(
        values=[ListItem(value=str(i**2)) for i in range(10)]
    )
    object_list.selected_cells = [
        (object_list.values[5], 'value'),
        (object_list.values[6], 'other value'),
        (object_list.values[8], 'value'),
    ]

    with store_exceptions_on_all_threads():
        ui = object_list.edit_traits(view=select_cells_view)
        editor = ui.get_editors('values')[0]
        gui.process_events()
        if is_current_backend_qt4():
            selected = editor.selected
        elif is_current_backend_wx():
            selected = editor.selected_cells

        press_ok_button(ui)
        gui.process_events()

    assert selected == [
        (object_list.values[5], 'value'),
        (object_list.values[6], 'other value'),
        (object_list.values[8], 'value'),
    ]


@skip_if_null
def test_table_editor_select_cell_index():
    gui = GUI()
    object_list = ObjectListWithSelection(
        values=[ListItem(value=str(i**2)) for i in range(10)]
    )
    object_list.selected_cell_index = (5, 1)

    with store_exceptions_on_all_threads():
        ui = object_list.edit_traits(view=select_cell_index_view)
        editor = ui.get_editors('values')[0]
        gui.process_events()
        if is_current_backend_qt4():
            selected = editor.selected_indices
        elif is_current_backend_wx():
            selected = editor.selected_cell_index

        press_ok_button(ui)
        gui.process_events()

    assert selected == (5, 1)


@skip_if_null
def test_table_editor_select_cell_indices():
    gui = GUI()
    object_list = ObjectListWithSelection(
        values=[ListItem(value=str(i**2)) for i in range(10)]
    )
    object_list.selected_cell_indices = [(5, 0), (6, 1), (8, 0)]

    with store_exceptions_on_all_threads():
        ui = object_list.edit_traits(view=select_cell_indices_view)
        editor = ui.get_editors('values')[0]
        gui.process_events()
        if is_current_backend_qt4():
            selected = editor.selected_indices
        elif is_current_backend_wx():
            selected = editor.selected_cell_indices

        press_ok_button(ui)
        gui.process_events()

    assert selected == [(5, 0), (6, 1), (8, 0)]


@skip_if_not_qt4
def test_progress_column():
    from traitsui.extras.progress_column import ProgressColumn
    progress_view = View(
        Item(
            'values',
            show_label=False,
            editor=TableEditor(
                columns=[
                    ObjectColumn(name='value'),
                    ProgressColumn(name='other_value'),
                ],
            )
        ),
        buttons=['OK'],
    )
    gui = GUI()
    object_list = ObjectList(
        values=[ListItem(value=str(i**2)) for i in range(10)]
    )

    with store_exceptions_on_all_threads():
        ui = object_list.edit_traits(view=progress_view)
        gui.process_events()
        press_ok_button(ui)
        gui.process_events()



import platform
import unittest

from pyface.gui import GUI

from traits.api import Enum, HasTraits
from traitsui.api import EnumEditor, UItem, View
from traitsui.tests._tools import (
    is_current_backend_qt4, is_current_backend_wx, skip_if_null,
    store_exceptions_on_all_threads)


is_windows = (platform.system() == 'Windows')


class EnumModel(HasTraits):

    value = Enum('one', 'two', 'three', 'four')


simple_view = View(
    UItem('value', style='simple'),
    resizable=True
)


simple_evaluate_view = View(
    UItem(
        'value',
        editor=EnumEditor(evaluate=True, values=['one', 'two', 'three', 'four']),
        style='simple'),
    resizable=True
)


simple_evaluate_view_popup = View(
    UItem(
        'value',
        editor=EnumEditor(evaluate=True, values=['one', 'two', 'three', 'four']),
        style='simple'),
    resizable=True
)


def get_combobox_text(combobox):
    """ Return the text given a combobox control """
    if is_current_backend_wx():
        import wx
        if isinstance(combobox, wx.Choice):
            return combobox.GetString(combobox.GetSelection())
        else:
            return combobox.GetValue()
    elif is_current_backend_qt4():
        return combobox.currentText()


def set_combobox_text(combobox, text):
    """ Set the text given a combobox control """
    if is_current_backend_wx():
        import wx
        if isinstance(combobox, wx.Choice):
            event_type = wx.EVT_CHOICE.typeId
            event = wx.CommandEvent(event_type, combobox.GetId())
            event.SetString(text)
            wx.PostEvent(combobox, event)
        else:
            combobox.SetValue(text)
            event_type = wx.EVT_COMBOBOX.typeId
            event = wx.CommandEvent(event_type, combobox.GetId())
            event.SetString(text)
            wx.PostEvent(combobox, event)

    elif is_current_backend_qt4():
        combobox.setEditText(text)


def set_combobox_index(combobox, idx):
    """ Set the choice index given a combobox control and index number """
    if is_current_backend_wx():
        import wx
        if isinstance(combobox, wx.Choice):
            event_type = wx.EVT_CHOICE.typeId
        else:
            event_type = wx.EVT_COMBOBOX.typeId
        event = wx.CommandEvent(event_type, combobox.GetId())
        text = combobox.GetString(idx)
        event.SetString(text)
        event.SetInt(idx)
        wx.PostEvent(combobox, event)

    elif is_current_backend_qt4():
        combobox.setCurrentIndex(idx)


class TestEnumEditor(unittest.TestCase):

    def check_enum_text_update(self, view):
        gui = GUI()
        enum_edit = EnumModel()

        with store_exceptions_on_all_threads():
            ui = enum_edit.edit_traits(view=view)
            self.addCleanup(ui.dispose)

            gui.process_events()
            editor = ui.get_editors("value")[0]
            combobox = editor.control

            self.assertEqual(get_combobox_text(combobox), "one")

            enum_edit.value = "two"
            gui.process_events()

            self.assertEqual(get_combobox_text(combobox), "two")

    def check_enum_object_update(self, view):
        gui = GUI()
        enum_edit = EnumModel()

        with store_exceptions_on_all_threads():
            ui = enum_edit.edit_traits(view=view)
            self.addCleanup(ui.dispose)

            gui.process_events()
            editor = ui.get_editors("value")[0]
            combobox = editor.control

            self.assertEqual(enum_edit.value, "one")

            set_combobox_text(combobox, "two")
            gui.process_events()

            self.assertEqual(enum_edit.value, "two")

    def check_enum_index_update(self, view):
        gui = GUI()
        enum_edit = EnumModel()

        with store_exceptions_on_all_threads():
            ui = enum_edit.edit_traits(view=view)
            self.addCleanup(ui.dispose)

            gui.process_events()
            editor = ui.get_editors("value")[0]
            combobox = editor.control

            self.assertEqual(enum_edit.value, "one")

            set_combobox_index(combobox, 1)
            gui.process_events()

            self.assertEqual(enum_edit.value, "two")

    def check_enum_text_bad_update(self, view):
        gui = GUI()
        enum_edit = EnumModel()

        with store_exceptions_on_all_threads():
            ui = enum_edit.edit_traits(view=view)
            self.addCleanup(ui.dispose)

            gui.process_events()
            editor = ui.get_editors("value")[0]
            combobox = editor.control

            self.assertEqual(enum_edit.value, "one")

            set_combobox_text(combobox, "t")
            gui.process_events()

            self.assertEqual(enum_edit.value, "one")

    @skip_if_null
    def test_simple_enum_editor_text(self):
        self.check_enum_text_update(simple_view)

    @skip_if_null
    def test_simple_enum_editor_index(self):
        self.check_enum_index_update(simple_view)

    @skip_if_null
    @unittest.skipIf(is_windows, "Test needs fixing on windows")
    def test_simple_evaluate_editor_text(self):
        self.check_enum_text_update(simple_evaluate_view)

    @skip_if_null
    @unittest.skipIf(is_windows, "Test needs fixing on windows")
    def test_simple_evaluate_editor_index(self):
        self.check_enum_index_update(simple_evaluate_view)

    @skip_if_null
    def test_simple_evaluate_editor_bad_text(self):
        self.check_enum_text_bad_update(simple_evaluate_view)

    @skip_if_null
    @unittest.skipIf(is_windows, "Test needs fixing on windows")
    def test_simple_evaluate_editor_object(self):
        self.check_enum_object_update(simple_evaluate_view)

    @skip_if_null
    @unittest.skipIf(is_windows, "Test needs fixing on windows")
    def test_simple_evaluate_popup_editor_text(self):
        self.check_enum_text_update(simple_evaluate_view_popup)

    @skip_if_null
    @unittest.skipIf(is_windows, "Test needs fixing on windows")
    def test_simple_evaluate_popup_editor_index(self):
        self.check_enum_index_update(simple_evaluate_view_popup)

    @skip_if_null
    def test_simple_evaluate_popup_editor_bad_text(self):
        self.check_enum_text_bad_update(simple_evaluate_view_popup)

    @skip_if_null
    @unittest.skipIf(is_windows, "Test needs fixing on windows")
    def test_simple_evaluate_popup_editor_object(self):
        self.check_enum_object_update(simple_evaluate_view_popup)



#-------------------------------------------------------------------------
#
#  Copyright (c) 2007, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in enthought/LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#  Author: David C. Morrill
#  Date:   03/30/2007
#
#  fixme:
#  - Get custom tree view images.
#  - Write a program to create a directory structure from a lesson plan file.
#
#-------------------------------------------------------------------------

""" A framework for creating interactive Python tutorials.
"""

#-------------------------------------------------------------------------
#  Imports:
#-------------------------------------------------------------------------

from __future__ import absolute_import, print_function

import sys
from operator import itemgetter
import os
import re

from string \
    import capwords

from traits.api \
    import HasPrivateTraits, HasTraits, File, Directory, Instance, Int, Str, \
    List, Bool, Dict, Any, Property, Delegate, Button, cached_property

from traitsui.api \
    import View, VGroup, HGroup, VSplit, HSplit, Tabbed, Item, Heading, \
    Handler, ListEditor, CodeEditor, EnumEditor, HTMLEditor, \
    TreeEditor, TitleEditor, ValueEditor, ShellEditor

from traitsui.menu \
    import NoButtons

from traitsui.tree_node \
    import TreeNode

from pyface.image_resource \
    import ImageResource
from io import open

try:
    from traitsui.wx.extra.windows.ie_html_editor \
        import IEHTMLEditor

    from traitsui.wx.extra.windows.flash_editor \
        import FlashEditor
    wx_available = True
except:
    IEHTMLEditor = HTMLEditor
    FlashEditor = ValueEditor  # Set this to a random editor to prevent error.
    wx_available = False

#-------------------------------------------------------------------------
#  Constants:
#-------------------------------------------------------------------------

# Correct program usage information:
Usage = """
Correct usage is: tutor.py [root_dir]
where:
    root_dir = Path to root of the tutorial tree

If omitted, 'root_dir' defaults to the current directory."""

# The standard list editor used:
list_editor = ListEditor(
    use_notebook=True,
    deletable=False,
    page_name='.title',
    export='DockWindowShell',
    dock_style='fixed'
)

# The standard code snippet editor used:
snippet_editor = ListEditor(
    use_notebook=True,
    deletable=False,
    page_name='.title',
    export='DockWindowShell',
    dock_style='tab',
    selected='snippet'
)

# Regular expressions used to match section directories:
dir_pat1 = re.compile(r'^(\d\d\d\d)_(.*)$')
dir_pat2 = re.compile(r'^(.*)_(\d+\.\d+)$')

# Regular expression used to match section header in a Python source file:
section_pat1 = re.compile(r'^#-*\[(.*)\]')  # Normal
section_pat2 = re.compile(r'^#-*<(.*)>')    # Hidden
section_pat3 = re.compile(r'^#-*\((.*)\)')  # Description

# Regular expression used to extract item titles from URLs:
url_pat1 = re.compile(r'^(.*)\[(.*)\](.*)$')  # Normal

# Is this running on the Windows platform?
is_windows = (sys.platform in ('win32', 'win64'))

# Python file section types:
IsCode = 0
IsHiddenCode = 1
IsDescription = 2

# HTML template for a default lecture:
DefaultLecture = """<html>
  <head>
  </head>
  <body>
    <p>This section contains the following topics:</p>
    <ul>
    %s
    </ul>
  </body>
</html>
"""

# HTML template for displaying a .wmv/.avi movie file:
WMVMovieTemplate = """<html>
<head>
</head>
<body>
<p><object classid="clsid:22D6F312-B0F6-11D0-94AB-0080C74C7E95" codebase="http://activex.microsoft.com/activex/controls/mplayer/en/nsmp2inf.cab#Version=6,4,5,715">
<param name="FileName" value="%s">
<param name="AutoStart" value="true">
<param name="ShowTracker" value="true">
<param name="ShowControls" value="true">
<param name="ShowGotoBar" value="false">
<param name="ShowDisplay" value="false">
<param name="ShowStatusBar" value="false">
<param name="AutoSize" value="true">
<embed src="%s" AutoStart="true" ShowTracker="true" ShowControls="true" ShowGotoBar="false" ShowDisplay="false" ShowStatusBar="false" AutoSize="true" pluginspage="http://www.microsoft.com/windows/windowsmedia/download/"></object></p>
</body>
</html>
"""

# HTML template for displaying a QuickTime.mov movie file:
QTMovieTemplate = """<html>
<head>
</head>
<body>
<p><object classid="clsid:02BF25D5-8C17-4B23-BC80-D3488ABDDC6B" codebase="http://www.apple.com/qtactivex/qtplugin.cab" width="100%%" height="100%%">
<param name="src" value="file:///%s">
<param name="scale" value="aspect">
<param name="autoplay" value="true">
<param name="loop" value="false">
<param name="controller" value="true">
<embed src="file:///%s" width="100%%" height="100%%" scale="aspect" autoplay="true" loop="false" controller="true" pluginspage="http://www.apple.com/quicktime/download"></object></p>
</body>
</html>
"""

# HTML template for displaying an image file:
ImageTemplate = """<html>
<head>
</head>
<body>
<img src="%s">
</body>
</html>
"""

# HTML template for playing an MP3 audio file:
MP3Template = """<html>
<head>
<bgsound src="%s">
</head>
<body>
<p>&nbsp;</p>
</body>
</html>
"""

#-------------------------------------------------------------------------
#  Returns the contents of a specified text file (or None):
#-------------------------------------------------------------------------


def read_file(path, mode='rU', encoding='utf8'):
    """ Returns the contents of a specified text file (or None).

    """
    try:
        with open(path, mode, encoding=encoding) as fh:
            result = fh.read()
        return result
    except Exception:
        return None


#-------------------------------------------------------------------------
#  Creates a title from a specified string:
#-------------------------------------------------------------------------

def title_for(title):
    """ Creates a title from a specified string.
    """
    return capwords(title.replace('_', ' '))

#-------------------------------------------------------------------------
#  Returns a relative CSS style sheet path for a specified path and parent
#  section:
#-------------------------------------------------------------------------


def css_path_for(path, parent):
    """ Returns a relative CSS style sheet path for a specified path and parent
        section.
    """
    if os.path.isfile(os.path.join(path, 'default.css')):
        return 'default.css'

    if parent is not None:
        result = parent.css_path
        if result != '':
            if path != parent.path:
                result = os.path.join('..', result)

            return result

    return ''

#-------------------------------------------------------------------------
#  'StdOut' class:
#-------------------------------------------------------------------------


class StdOut(object):
    """ Simulate stdout, but redirect the output to the 'output' string
        supplied by some 'owner' object.
    """

    def __init__(self, owner):
        self.owner = owner

    def write(self, data):
        """ Adds the specified data to the output log.
        """
        self.owner.output += data

    def flush(self):
        """ Flushes all current data to the output log.
        """
        pass

#-------------------------------------------------------------------------
#  'NoDemo' class:
#-------------------------------------------------------------------------


class NoDemo(HasPrivateTraits):

    #-- Traits View Definitions ----------------------------------------------

    view = View(
        Heading('No demo defined for this lab.'),
        resizable=True
    )

#-------------------------------------------------------------------------
#  'DemoPane' class:
#-------------------------------------------------------------------------


class DemoPane(HasPrivateTraits):
    """ Displays the contents of a Python lab's *demo* value.
    """

    #-- Trait Definitions ----------------------------------------------------

    demo = Instance(HasTraits, factory=NoDemo)

    #-- Traits View Definitions ----------------------------------------------

    view = View(
        Item('demo',
             id='demo',
             show_label=False,
             style='custom',
             resizable=True
             ),
        id='enthought.tutor.demo',
        resizable=True
    )

#-------------------------------------------------------------------------
#  'ATutorialItem' class:
#-------------------------------------------------------------------------


class ATutorialItem(HasPrivateTraits):
    """ Defines the abstract base class for each type of item (HTML, Flash,
        text, code) displayed within the tutor.
    """

    #-- Traits Definitions ---------------------------------------------------

    # The title for the item:
    title = Str

    # The path to the item:
    path = File

    # The displayable content for the item:
    content = Property

#-------------------------------------------------------------------------
#  'ADescriptionItem' class:
#-------------------------------------------------------------------------


class ADescriptionItem(ATutorialItem):
    """ Defines a common base class for all description items.
    """

    #-- Event Handlers -------------------------------------------------------

    def _path_changed(self, path):
        """ Sets the title for the item based on the item's path name.
        """
        self.title = title_for(os.path.splitext(os.path.basename(
            path))[0])

#-------------------------------------------------------------------------
#  'HTMLItem' class:
#-------------------------------------------------------------------------


class HTMLItem(ADescriptionItem):
    """ Defines a class used for displaying a single HTML page within the tutor
        using the default Traits HTML editor.
    """

    #-- Traits Definitions ---------------------------------------------------

    url = Str

    #-- Traits View Definitions ----------------------------------------------

    view = View(
        Item('content',
             style='readonly',
             show_label=False,
             editor=HTMLEditor()
             )
    )

    #-- Event Handlers -------------------------------------------------------

    def _url_changed(self, url):
        """ Sets the item title when the 'url' is changed.
        """
        match = url_pat1.match(url)
        if match is not None:
            title = match.group(2).strip()
        else:
            title = url.strip()
            col = title.rfind('/')
            if col >= 0:
                title = os.path.splitext(title[col + 1:])[0]

        self.title = title

    #-- Property Implementations ---------------------------------------------

    @cached_property
    def _get_content(self):
        """ Returns the item content.
        """
        url = self.url
        if url != '':
            match = url_pat1.match(url)
            if match is not None:
                url = match.group(1) + match.group(3)

            return url

        return read_file(self.path)

    def _set_content(self, content):
        """ Sets the item content.
        """
        self._content = content

#-------------------------------------------------------------------------
#  'HTMLStrItem' class:
#-------------------------------------------------------------------------


class HTMLStrItem(HTMLItem):
    """ Defines a class used for displaying a single HTML text string within
        the tutor using the default Traits HTML editor.
    """

    # Make the content a real trait rather than a property:
    content = Str

#-------------------------------------------------------------------------
#  'IEHTMLItem' class:
#-------------------------------------------------------------------------


class IEHTMLItem(HTMLItem):
    """ Defines a class used for displaying a single HTML page within the tutor
        using the Traits Internet Explorer HTML editor.
    """

    #-- Traits View Definitions ----------------------------------------------

    view = View(
        Item('content',
             style='readonly',
             show_label=False,
             editor=IEHTMLEditor()
             )
    )

#-------------------------------------------------------------------------
#  'IEHTMLStrItem' class:
#-------------------------------------------------------------------------


class IEHTMLStrItem(IEHTMLItem):
    """ Defines a class used for displaying a single HTML text string within
        the tutor using the Traits Internet Explorer HTML editor.
    """

    # Make the content a real trait rather than a property:
    content = Str

#-------------------------------------------------------------------------
#  'FlashItem' class:
#-------------------------------------------------------------------------


class FlashItem(HTMLItem):
    """ Defines a class used for displaying a Flash-based animation or video
        within the tutor.
    """

    #-- Traits View Definitions ----------------------------------------------

    view = View(
        Item('content',
             style='readonly',
             show_label=False,
             editor=FlashEditor()
             )
    )

#-------------------------------------------------------------------------
#  'TextItem' class:
#-------------------------------------------------------------------------


class TextItem(ADescriptionItem):
    """ Defines a class used for displaying a text file within the tutor.
    """

    #-- Traits View Definitions ----------------------------------------------

    view = View(
        Item('content',
             style='readonly',
             show_label=False,
             editor=CodeEditor(show_line_numbers=False,
                               selected_color=0xFFFFFF)
             )
    )

    #-- Property Implementations ---------------------------------------------

    @cached_property
    def _get_content(self):
        """ Returns the item content.
        """
        return read_file(self.path)

#-------------------------------------------------------------------------
#  'TextStrItem' class:
#-------------------------------------------------------------------------


class TextStrItem(TextItem):
    """ Defines a class used for displaying a text file within the tutor.
    """

    # Make the content a real trait, rather than a property:
    content = Str

#-------------------------------------------------------------------------
#  'CodeItem' class:
#-------------------------------------------------------------------------


class CodeItem(ATutorialItem):
    """ Defines a class used for displaying a Python source code fragment
        within the tutor.
    """

    #-- Trait Definitions ----------------------------------------------------

    # The displayable content for the item (override):
    content = Str

    # The starting line of the code snippet within the original file:
    start_line = Int

    # The currently selected line:
    selected_line = Int

    # Should this section normally be hidden?
    hidden = Bool

    #-- Traits View Definitions ----------------------------------------------

    view = View(
        Item('content',
             style='custom',
             show_label=False,
             editor=CodeEditor(selected_line='selected_line')
             )
    )

#-------------------------------------------------------------------------
#  'ASection' abstract base class:
#-------------------------------------------------------------------------


class ASection(HasPrivateTraits):
    """ Defines an abstract base class for a single section of a tutorial.
    """

    #-- Traits Definitions ---------------------------------------------------

    # The title of the section:
    title = Str

    # The path to this section:
    path = Directory

    # The parent section of this section (if any):
    parent = Instance('ASection')

    # Optional table of contents (can be used to define/locate the
    # subsections):
    toc = List(Str)

    # The path to the CSS style sheet to use for this section:
    css_path = Property

    # The list of subsections contained in this section:
    subsections = Property  # List( ASection )

    # This section can be executed:
    is_runnable = Bool(True)

    # Should the Python code be automatically executed on start-up?
    auto_run = Bool(False)

    #-- Property Implementations ---------------------------------------------

    @cached_property
    def _get_subsections(self):
        """ Returns the subsections for this section:
        """
        if len(self.toc) > 0:
            self._load_toc()
        else:
            self._load_dirs()

        # Return the cached list of sections:
        return self._subsections

    @cached_property
    def _get_css_path(self):
        """ Returns the path to the CSS style sheet for this section.
        """
        return css_path_for(self.path, self.parent)

    #-- Private Methods ------------------------------------------------------

    def _load_dirs(self):
        """ Defines the section's subsections by analyzing all of the section's
            sub-directories.
        """
        # No value cached yet:
        dirs = []
        path = self.path

        # Find every sub-directory whose name begins with a number of the
        # form ddd, or ends with a number of the form _ddd.ddd (used for
        # sorting them into the correct presentation order):
        for name in os.listdir(path):
            dir = os.path.join(path, name)
            if os.path.isdir(dir):
                match = dir_pat1.match(name)
                if match is not None:
                    dirs.append((float(match.group(1)),
                                 match.group(2), dir))
                else:
                    match = dir_pat2.match(name)
                    if match is not None:
                        dirs.append((float(match.group(2)),
                                     match.group(1), dir))

        # Sort the directories by their index value:
        dirs.sort(key=itemgetter(0))

        # Create the appropriate type of section for each valid directory:
        self._subsections = [
            sf.section for sf in [
                SectionFactory(title=title_for(title),
                               parent=self).trait_set(
                    path=dir)
                for index, title, dir in dirs
            ] if sf.section is not None
        ]

    def _load_toc(self):
        """ Defines the section's subsections by finding matches for the items
            defined in the section's table of contents.
        """
        toc = self.toc
        base_names = [item.split(':', 1)[0] for item in toc]
        subsections = [None] * len(base_names)
        path = self.path

        # Classify all file names that match a base name in the table of
        # contents:
        for name in os.listdir(path):
            try:
                base_name = os.path.splitext(os.path.basename(name))[0]
                index = base_names.index(base_name)
                if subsections[index] is None:
                    subsections[index] = []
                subsections[index].append(name)
            except:
                pass

        # Try to convert each group of names into a section:
        for i, names in enumerate(subsections):

            # Only process items for which we found at least one matching file
            # name:
            if names is not None:

                # Get the title for the section from its table of contents
                # entry:
                parts = toc[i].split(':', 1)
                if len(parts) == 1:
                    title = title_for(parts[0].strip())
                else:
                    title = parts[1].strip()

                # Handle an item with one file which is a directory as a normal
                # section:
                if len(names) == 1:
                    dir = os.path.join(path, names[0])
                    if os.path.isdir(dir):
                        subsections[i] = SectionFactory(title=title,
                                                        parent=self).trait_set(
                            path=dir).section
                        continue

                # Otherwise, create a section from the list of matching files:
                subsections[i] = SectionFactory(title=title,
                                                parent=self,
                                                files=names).trait_set(
                    path=path).section

        # Set the subsections to the non-None values that are left:
        self._subsections = [subsection for subsection in subsections
                             if subsection is not None]

#-------------------------------------------------------------------------
#  'Lecture' class:
#-------------------------------------------------------------------------


class Lecture(ASection):
    """ Defines a lecture, which is a section of a tutorial with descriptive
        information, but no associated Python code. Can be used to provide
        course overviews, introductory sections, or lead-ins to follow-on
        lessons or labs.
    """

    #-- Trait Definitions-----------------------------------------------------

    # The list of descriptive items for the lecture:
    descriptions = List(ATutorialItem)

    # This section can be executed (override):
    is_runnable = False

    #-- Traits View Definitions ----------------------------------------------

    view = View(
        Item('descriptions',
             style='custom',
             show_label=False,
             editor=list_editor
             ),
        id='enthought.tutor.lecture'
    )

#-------------------------------------------------------------------------
#  'LabHandler' class:
#-------------------------------------------------------------------------


class LabHandler(Handler):
    """ Defines the controller functions for the Lab view.
    """

    def init(self, info):
        """ Handles initialization of the view.
        """
        # Run the associated Python code if the 'auto-run' feature is enabled:
        if info.object.auto_run:
            info.object.run_code()

#-------------------------------------------------------------------------
#  'Lab' class:
#-------------------------------------------------------------------------


class Lab(ASection):
    """ Defines a lab, which is a section of a tutorial with only Python code.
        This type of section might typically follow a lecture which introduced
        the code being worked on in the lab.
    """

    #-- Trait Definitions-----------------------------------------------------

    # The set-up code (if any) for the lab:
    setup = Instance(CodeItem)

    # The list of code items for the lab:
    snippets = List(CodeItem)

    # The list of visible code items for the lab:
    visible_snippets = Property(depends_on='visible', cached=True)

    # The currently selected snippet:
    snippet = Instance(CodeItem)

    # Should normally hidden code items be shown?
    visible = Bool(False)

    # The dictionary containing the items from the Python code execution:
    values = Dict  # Any( {} )

    # The run Python code button:
    run = Button(image=ImageResource('run'), height_padding=1)

    # User error message:
    message = Str

    # The output produced while the program is running:
    output = Str

    # The current demo pane (if any):
    demo = Instance(DemoPane, ())

    #-- Traits View Definitions ----------------------------------------------

    view = View(
        VSplit(
            VGroup(
                Item('visible_snippets',
                     style='custom',
                     show_label=False,
                     editor=snippet_editor
                     ),
                HGroup(
                    Item('run',
                         style='custom',
                         show_label=False,
                         tooltip='Run the Python code'
                         ),
                    '_',
                    Item('message',
                         springy=True,
                         show_label=False,
                         editor=TitleEditor()
                         ),
                    '_',
                    Item('visible',
                         label='View hidden sections'
                         )
                ),
            ),
            Tabbed(
                Item('values',
                     id='values_1',
                     label='Shell',
                     editor=ShellEditor(share=True),
                     dock='tab',
                     export='DockWindowShell'
                     ),
                Item('values',
                     id='values_2',
                     editor=ValueEditor(),
                     dock='tab',
                     export='DockWindowShell'
                     ),
                Item('output',
                     style='readonly',
                     editor=CodeEditor(show_line_numbers=False,
                                       selected_color=0xFFFFFF),
                     dock='tab',
                     export='DockWindowShell'
                     ),
                Item('demo',
                     id='demo',
                     style='custom',
                     resizable=True,
                     dock='tab',
                     export='DockWindowShell'
                     ),
                show_labels=False,
            ),
            id='splitter',
        ),
        id='enthought.tutor.lab',
        handler=LabHandler
    )

    #-- Event Handlers -------------------------------------------------------

    def _run_changed(self):
        """ Runs the current set of snippet code.
        """
        self.run_code()

    #-- Property Implementations ---------------------------------------------

    @cached_property
    def _get_visible_snippets(self):
        """ Returns the list of code items that are currently visible.
        """
        if self.visible:
            return self.snippets

        return [snippet for snippet in self.snippets if (not snippet.hidden)]

    #-- Public Methods -------------------------------------------------------

    def run_code(self):
        """ Runs all of the code snippets associated with the section.
        """
        # Reconstruct the lab code from the current set of code snippets:
        start_line = 1
        module = ''
        for snippet in self.snippets:
            snippet.start_line = start_line
            module = '%s\n\n%s' % (module, snippet.content)
            start_line += (snippet.content.count('\n') + 2)

        # Reset any syntax error and message log values:
        self.message = self.output = ''

        # Redirect standard out and error to the message log:
        stdout, stderr = sys.stdout, sys.stderr
        sys.stdout = sys.stderr = StdOut(self)

        try:
            try:
                # Get the execution context dictionary:
                values = self.values

                # Clear out any special variables defined by the last run:
                for name in ('demo', 'popup'):
                    if isinstance(values.get(name), HasTraits):
                        del values[name]

                # Execute the current lab code:
                exec(module[2:], values, values)

                # fixme: Hack trying to update the Traits UI view of the dict.
                self.values = {}
                self.values = values

                # Handle a 'demo' value being defined:
                demo = values.get('demo')
                if not isinstance(demo, HasTraits):
                    demo = NoDemo()
                self.demo.demo = demo

                # Handle a 'popup' value being defined:
                popup = values.get('popup')
                if isinstance(popup, HasTraits):
                    popup.edit_traits(kind='livemodal')

            except SyntaxError as excp:
                # Convert the line number of the syntax error from one in the
                # composite module to one in the appropriate code snippet:
                line = excp.lineno
                if line is not None:
                    snippet = self.snippets[0]
                    for s in self.snippets:
                        if s.start_line > line:
                            break
                        snippet = s
                    line -= (snippet.start_line - 1)

                    # Highlight the line in error:
                    snippet.selected_line = line

                    # Select the correct code snippet:
                    self.snippet = snippet

                    # Display the syntax error message:
                    self.message = '%s in column %s of line %s' % (
                                   excp.msg.capitalize(), excp.offset, line)
                else:
                    # Display the syntax error message without line # info:
                    self.message = excp.msg.capitalize()
            except:
                import traceback
                traceback.print_exc()
        finally:
            # Restore standard out and error to their original values:
            sys.stdout, sys.stderr = stdout, stderr

#-------------------------------------------------------------------------
#  'Lesson' class:
#-------------------------------------------------------------------------


class Lesson(Lab):
    """ Defines a lesson, which is a section of a tutorial with both descriptive
        information and associated Python code.
    """

    #-- Trait Definitions-----------------------------------------------------

    # The list of descriptive items for the lesson:
    descriptions = List(ATutorialItem)

    #-- Traits View Definitions ----------------------------------------------

    view = View(
        HSplit(
            Item('descriptions',
                 label='Lesson',
                 style='custom',
                 show_label=False,
                 dock='horizontal',
                 editor=list_editor
                 ),
            VSplit(
                VGroup(
                    Item('visible_snippets',
                         style='custom',
                         show_label=False,
                         editor=snippet_editor
                         ),
                    HGroup(
                        Item('run',
                             style='custom',
                             show_label=False,
                             tooltip='Run the Python code'
                             ),
                        '_',
                        Item('message',
                             springy=True,
                             show_label=False,
                             editor=TitleEditor()
                             ),
                        '_',
                        Item('visible',
                             label='View hidden sections'
                             )
                    ),
                    label='Lab',
                    dock='horizontal'
                ),
                Tabbed(
                    Item('values',
                         id='values_1',
                         label='Shell',
                         editor=ShellEditor(share=True),
                         dock='tab',
                         export='DockWindowShell'

                         ),
                    Item('values',
                         id='values_2',
                         editor=ValueEditor(),
                         dock='tab',
                         export='DockWindowShell'
                         ),
                    Item('output',
                         style='readonly',
                         editor=CodeEditor(show_line_numbers=False,
                                           selected_color=0xFFFFFF),
                         dock='tab',
                         export='DockWindowShell'
                         ),
                    Item('demo',
                         id='demo',
                         style='custom',
                         resizable=True,
                         dock='tab',
                         export='DockWindowShell'
                         ),
                    show_labels=False,
                ),
                label='Lab',
                dock='horizontal'
            ),
            id='splitter',
        ),
        id='enthought.tutor.lesson',
        handler=LabHandler
    )

#-------------------------------------------------------------------------
#  'Demo' class:
#-------------------------------------------------------------------------


class Demo(Lesson):
    """ Defines a demo, which is a section of a tutorial with both descriptive
        information and associated Python code which is executed but not
        shown.
    """

    #-- Traits View Definitions ----------------------------------------------

    view = View(
        HSplit(
            Item('descriptions',
                 label='Lesson',
                 style='custom',
                 show_label=False,
                 dock='horizontal',
                 editor=list_editor
                 ),
            Item('demo',
                 id='demo',
                 style='custom',
                 show_label=False,
                 resizable=True,
                 dock='horizontal',
                 export='DockWindowShell'
                 ),
            id='splitter',
        ),
        id='enthought.tutor.demo',
        handler=LabHandler
    )

#-------------------------------------------------------------------------
#  'SectionFactory' class:
#-------------------------------------------------------------------------


class SectionFactory(HasPrivateTraits):
    """ Defines a class that creates Lecture, Lesson or Lab sections (or None),
        based on the content of a specified directory. None is returned if the
        directory does not contain any recognized files.
    """

    #-- Traits Definitions ---------------------------------------------------

    # The path the section is to be created for:
    path = Directory

    # The list of files contained in the section:
    files = List(Str)

    # The parent of the section being created:
    parent = Instance(ASection)

    # The section created from the path:
    section = Instance(ASection)

    # The title for the section:
    title = Str

    # The optional table of contents for the section:
    toc = List(Str)

    # The list of descriptive items for the section:
    descriptions = List(ADescriptionItem)

    # The list of code snippet items for the section:
    snippets = List(CodeItem)

    # The path to the CSS style sheet for the section:
    css_path = Property

    # Should the Python code be automatically executed on start-up?
    auto_run = Bool(False)

    #-- Event Handlers -------------------------------------------------------

    def _path_changed(self, path):
        """ Creates the appropriate section based on the value of the path.
        """
        # Get the list of files to process:
        files = self.files
        if len(files) == 0:
            # If none were specified, then use all files in the directory:
            files = os.listdir(path)

            # Process the description file (if any) first:
            for name in files:
                if os.path.splitext(name)[1] == '.desc':
                    self._add_desc_item(os.path.join(path, name))
                    break

        # Try to convert each file into one or more 'xxxItem' objects:
        toc = [item.split(':', 1)[0].strip() for item in self.toc]
        for name in files:
            file_name = os.path.join(path, name)

            # Only process the ones that are actual files:
            if os.path.isfile(file_name):

                # Use the file extension to determine the file's type:
                root, ext = os.path.splitext(name)
                if (root not in toc) and (len(ext) > 1):

                    # If we have a handler for the file type, invoke it:
                    method = getattr(self, '_add_%s_item' % ext[1:].lower(),
                                     None)
                    if method is not None:
                        method(file_name)

        # Based on the type of items created (if any), create the corresponding
        # type of section:
        if len(self.descriptions) > 0:
            if len(self.snippets) > 0:
                if len([snippet for snippet in self.snippets
                        if (not snippet.hidden)]) > 0:
                    self.section = Lesson(
                        title=self.title,
                        path=path,
                        toc=self.toc,
                        parent=self.parent,
                        descriptions=self.descriptions,
                        snippets=self.snippets,
                        auto_run=self.auto_run
                    )
                else:
                    self.section = Demo(
                        title=self.title,
                        path=path,
                        toc=self.toc,
                        parent=self.parent,
                        descriptions=self.descriptions,
                        snippets=self.snippets,
                        auto_run=True
                    )
            else:
                self.section = Lecture(
                    title=self.title,
                    path=path,
                    toc=self.toc,
                    parent=self.parent,
                    descriptions=self.descriptions
                )
        elif len(self.snippets) > 0:
            self.section = Lab(
                title=self.title,
                path=path,
                toc=self.toc,
                parent=self.parent,
                snippets=self.snippets,
                auto_run=self.auto_run
            )
        else:
            # No descriptions or code snippets were found. Create a lecture
            # anyway:
            section = Lecture(
                title=self.title,
                path=path,
                toc=self.toc,
                parent=self.parent
            )

            # If the lecture has subsections, then return the lecture and add
            # a default item containing a description of the subsections of the
            # lecture:
            if len(section.subsections) > 0:
                self._create_html_item(path=path, content=DefaultLecture % ('\n'.join(
                    ['<li>%s</li>' % subsection.title
                     for subsection in section.subsections])))
                section.descriptions = self.descriptions
                self.section = section

    #-- Property Implementations ---------------------------------------------

    def _get_css_path(self):
        """ Returns the path to the CSS style sheet for the section.
        """
        return css_path_for(self.path, self.parent)

    #-- Factory Methods for Creating Section Items Based on File Type --------

    def _add_py_item(self, path):
        """ Creates the code snippets for a Python source file.
        """
        source = read_file(path)
        if source is not None:
            lines = source.replace('\r', '').split('\n')
            start_line = 0
            title = 'Prologue'
            type = IsCode

            for i, line in enumerate(lines):
                match = section_pat1.match(line)
                if match is not None:
                    next_type = IsCode
                else:
                    match = section_pat2.match(line)
                    if match is not None:
                        next_type = IsHiddenCode
                    else:
                        next_type = IsDescription
                        match = section_pat3.match(line)

                if match is not None:
                    self._add_snippet(title, path, lines, start_line, i - 1,
                                      type)
                    start_line = i + 1
                    title = match.group(1).strip()
                    type = next_type

            self._add_snippet(title, path, lines, start_line, i, type)

    def _add_txt_item(self, path):
        """ Creates a description item for a normal text file.
        """
        self.descriptions.append(TextItem(path=path))

    def _add_htm_item(self, path):
        """ Creates a description item for an HTML file.
        """
        # Check if there is a corresponding .rst (restructured text) file:
        dir, base_name = os.path.split(path)
        rst = os.path.join(dir, os.path.splitext(base_name)[0] + '.rst')

        # If no .rst file exists, just add the file as a normal HTML file:
        if not os.path.isfile(rst):
            self._create_html_item(path=path)

    def _add_html_item(self, path):
        """ Creates a description item for an HTML file.
        """
        self._add_htm_item(path)

    def _add_url_item(self, path):
        """ Creates a description item for a file containing URLs.
        """
        data = read_file(path)
        if data is not None:
            for url in [line for line in data.split('\n')
                        if line.strip()[:1] not in ('', '#')]:
                self._create_html_item(url=url.strip())

    def _add_rst_item(self, path):
        """ Creates a description item for a ReSTructured text file.
        """
        # If docutils is not installed, just process the file as an ordinary
        # text file:
        try:
            from docutils.core import publish_cmdline
        except:
            self._add_txt_item(path)
            return

        # Get the name of the HTML file we will write to:
        dir, base_name = os.path.split(path)
        html = os.path.join(dir, os.path.splitext(base_name)[0] + '.htm')

        # Try to find a CSS style sheet, and set up the docutil overrides if
        # found:
        settings = {}
        css_path = self.css_path
        if css_path != '':
            css_path = os.path.join(self.path, css_path)
            settings['stylesheet_path'] = css_path
            settings['embed_stylesheet'] = True
            settings['stylesheet'] = None
        else:
            css_path = path

        # If the HTML file does not exist, or is older than the restructured
        # text file, then let docutils convert it to HTML:
        is_file = os.path.isfile(html)
        if ((not is_file) or
            (os.path.getmtime(path) > os.path.getmtime(html)) or
                (os.path.getmtime(css_path) > os.path.getmtime(html))):

            # Delete the current HTML file (if any):
            if is_file:
                os.remove(html)

            # Let docutils create a new HTML file from the restructured text
            # file:
            publish_cmdline(writer_name='html',
                            argv=[path, html],
                            settings_overrides=settings)

        if os.path.isfile(html):
            # If there is now a valid HTML file, use it:
            self._create_html_item(path=html)

        else:
            # Otherwise, just use the original restructured text file:
            self._add_txt_item(path)

    def _add_swf_item(self, path):
        """ Creates a description item for a Flash file.
        """
        if is_windows and wx_available:
            self.descriptions.append(FlashItem(path=path))

    def _add_mov_item(self, path):
        """ Creates a description item for a QuickTime movie file.
        """
        path2 = path.replace(':', '|')
        self._create_html_item(path=path,
                               content=QTMovieTemplate % (path2, path2))

    def _add_wmv_item(self, path):
        """ Creates a description item for a Windows movie file.
        """
        self._create_html_item(path=path,
                               content=WMVMovieTemplate % (path, path))

    def _add_avi_item(self, path):
        """ Creates a description item for an AVI movie file.
        """
        self._add_wmv_item(path)

    def _add_jpg_item(self, path):
        """ Creates a description item for a JPEG image file.
        """
        self._create_html_item(path=path,
                               content=ImageTemplate % path)

    def _add_jpeg_item(self, path):
        """ Creates a description item for a JPEG image file.
        """
        self._add_jpg_item(path)

    def _add_png_item(self, path):
        """ Creates a description item for a PNG image file.
        """
        self._add_jpg_item(path)

    def _add_mp3_item(self, path):
        """ Creates a description item for an mp3 audio file.
        """
        self._create_html_item(path=path,
                               content=MP3Template % path)

    def _add_desc_item(self, path):
        """ Creates a section title from a description file.
        """
        # If we've already processed a description file, then we're done:
        if len(self.toc) > 0:
            return

        lines = []
        desc = read_file(path)
        if desc is not None:
            # Split the file into lines and save the non-empty, non-comment
            # lines:
            for line in desc.split('\n'):
                line = line.strip()
                if (len(line) > 0) and (line[0] != '#'):
                    lines.append(line)

        if len(lines) == 0:
            # If the file didn't have anything useful in it, set a title based
            # on the description file name:
            self.title = title_for(
                os.path.splitext(os.path.basename(path))[0])
        else:
            # Otherwise, set the title and table of contents from the lines in
            # the file:
            self.title = lines[0]
            self.toc = lines[1:]

    #-- Private Methods ------------------------------------------------------

    def _add_snippet(self, title, path, lines, start_line, end_line, type):
        """ Adds a new code snippet or restructured text item to the list of
            code snippet or description items.
        """
        # Trim leading and trailing blank lines from the snippet:
        while start_line <= end_line:
            if lines[start_line].strip() != '':
                break
            start_line += 1

        while end_line >= start_line:
            if lines[end_line].strip() != '':
                break
            end_line -= 1

        # Only add if the snippet is not empty:
        if start_line <= end_line:

            # Check for the title containing the 'auto-run' flag ('*'):
            if title[:1] == '*':
                self.auto_run = True
                title = title[1:].strip()

            if title[-1:] == '*':
                self.auto_run = True
                title = title[:-1].strip()

            # Extract out just the lines we will use:
            content_lines = lines[start_line: end_line + 1]

            if type == IsDescription:
                # Add the new restructured text description:
                self._add_description(content_lines, title)
            else:
                # Add the new code snippet:
                self.snippets.append(CodeItem(
                    title=title or 'Code',
                    path=path,
                    hidden=(type == IsHiddenCode),
                    content='\n'.join(content_lines)
                ))

    def _add_description(self, lines, title):
        """ Converts a restructured text string to HTML and adds it as
            description item.
        """
        # Scan the lines for any imbedded Python code that should be shown as
        # a separate snippet:
        i = 0
        while i < len(lines):
            if lines[i].strip()[-2:] == '::':
                i = self._check_embedded_code(lines, i + 1)
            else:
                i += 1

        # Strip off any docstring style triple quotes (if necessary):
        content = '\n'.join(lines).strip()
        if content[:3] in ( '"""', "'''" ):
            content = content[3:]

        if content[-3:] in ( '"""', "'''" ):
            content = content[:-3]

        content = content.strip()

        # If docutils is not installed, just add it as a text string item:
        try:
            from docutils.core import publish_string
        except:
            self.descriptions.append(TextStrItem(content=content,
                                                 title=title))
            return

        # Try to find a CSS style sheet, and set up the docutil overrides if
        # found:
        settings = {}
        css_path = self.css_path
        if css_path != '':
            css_path = os.path.join(self.path, css_path)
            settings['stylesheet_path'] = css_path
            settings['embed_stylesheet'] = True
            settings['stylesheet'] = None

        # Convert it from restructured text to HTML:
        html = publish_string(content, writer_name='html',
                              settings_overrides=settings)

        # Choose the right HTML renderer:
        if is_windows and wx_available:
            item = IEHTMLStrItem(content=html, title=title)
        else:
            item = HTMLStrItem(content=html, title=title)

        # Add the resulting item to the descriptions list:
        self.descriptions.append(item)

    def _create_html_item(self, **traits):
        """ Creates a platform specific html item and adds it to the list of
            descriptions.
        """
        if is_windows and wx_available:
            item = IEHTMLItem(**traits)
        else:
            item = HTMLItem(**traits)

        self.descriptions.append(item)

    def _check_embedded_code(self, lines, start):
        """ Checks for an embedded Python code snippet within a description.
        """
        n = len(lines)
        while start < n:
            line = lines[start].strip()

            if line == '':
                start += 1
                continue

            if (line[:1] != '[') or (line[-1:] != ']'):
                break

            del lines[start]

            n -= 1
            title = line[1:-1].strip()
            line = lines[start] + '.'
            pad = len(line) - len(line.strip())
            clines = []

            while start < n:
                line = lines[start] + '.'
                len_line = len(line.strip())
                if (len_line > 1) and ((len(line) - len_line) < pad):
                    break

                if (len(clines) > 0) or (len_line > 1):
                    clines.append(line[pad: -1])

                start += 1

            # Add the new code snippet:
            self.snippets.append(CodeItem(
                title=title or 'Code',
                content='\n'.join(clines)
            ))

            break

        return start

#-------------------------------------------------------------------------
#  Tutor tree editor:
#-------------------------------------------------------------------------

tree_editor = TreeEditor(
    nodes=[
        TreeNode(
            children='subsections',
            label='title',
            rename=False,
            copy=False,
            delete=False,
            delete_me=False,
            insert=False,
            auto_open=True,
            auto_close=False,
            node_for=[ASection],
            icon_group='<group>'
        )
    ],
    editable=False,
    auto_open=1,
    selected='section'
)

#-------------------------------------------------------------------------
#  'Tutor' class:
#-------------------------------------------------------------------------


class Tutor(HasPrivateTraits):
    """ The main tutorial class which manages the presentation and navigation
        of the entire tutorial.
    """

    #-- Trait Definitions ----------------------------------------------------

    # The path to the files distributed with the tutor:
    home = Directory

    # The path to the root of the tutorial tree:
    path = Directory

    # The root of the tutorial lesson tree:
    root = Instance(ASection)

    # The current section of the tutorial being displayed:
    section = Instance(ASection)

    # The next section:
    next_section = Property(depends_on='section', cached=True)

    # The previous section:
    previous_section = Property(depends_on='section', cached=True)

    # The previous section button:
    previous = Button(image=ImageResource('previous'), height_padding=1)

    # The next section button:
    next = Button(image=ImageResource('next'), height_padding=1)

    # The parent section button:
    parent = Button(image=ImageResource('parent'), height_padding=1)

    # The reload tutor button:
    reload = Button(image=ImageResource('reload'), height_padding=1)

    # The title of the current session:
    title = Property(depends_on='section')

    #-- Traits View Definitions ----------------------------------------------

    view = View(
        VGroup(
            HGroup(
                Item('previous',
                     style='custom',
                     enabled_when='previous_section is not None',
                     tooltip='Go to previous section'
                     ),
                Item('parent',
                     style='custom',
                     enabled_when='(section is not None) and '
                     '(section.parent is not None)',
                     tooltip='Go up one level'
                     ),
                Item('next',
                     style='custom',
                     enabled_when='next_section is not None',
                     tooltip='Go to next section'
                     ),
                '_',
                Item('title',
                     springy=True,
                     editor=TitleEditor()
                     ),
                '_',
                Item('reload',
                     style='custom',
                     tooltip='Reload the tutorial'
                     ),
                show_labels=False
            ),
            '_',
            HSplit(
                Item('root',
                     label='Table of Contents',
                     editor=tree_editor,
                     dock='horizontal',
                     export='DockWindowShell'
                     ),
                Item('section',
                     id='section',
                     label='Current Lesson',
                     style='custom',
                     resizable=True,
                     dock='horizontal'
                     ),
                id='splitter',
                show_labels=False
            )
        ),
        title='Python Tutor',
        id='dmorrill.tutor.tutor:1.0',
        buttons=NoButtons,
        resizable=True,
        width=0.8,
        height=0.8
    )

    #-- Event Handlers -------------------------------------------------------

    def _path_changed(self, path):
        """ Handles the tutorial root path being changed.
        """
        self.init_tutor()

    def _next_changed(self):
        """ Displays the next tutorial section.
        """
        self.section = self.next_section

    def _previous_changed(self):
        """ Displays the previous tutorial section.
        """
        self.section = self.previous_section

    def _parent_changed(self):
        """ Displays the parent of the current tutorial section.
        """
        self.section = self.section.parent

    def _reload_changed(self):
        """ Reloads the tutor from the original path specified.
        """
        self.init_tutor()

    #-- Property Implementations ---------------------------------------------

    @cached_property
    def _get_next_section(self):
        """ Returns the next section of the tutorial.
        """
        next = None
        section = self.section
        if len(section.subsections) > 0:
            next = section.subsections[0]
        else:
            parent = section.parent
            while parent is not None:
                index = parent.subsections.index(section)
                if index < (len(parent.subsections) - 1):
                    next = parent.subsections[index + 1]
                    break

                parent, section = parent.parent, parent

        return next

    @cached_property
    def _get_previous_section(self):
        """ Returns the previous section of the tutorial.
        """
        previous = None
        section = self.section
        parent = section.parent
        if parent is not None:
            index = parent.subsections.index(section)
            if index > 0:
                previous = parent.subsections[index - 1]
                while len(previous.subsections) > 0:
                    previous = previous.subsections[-1]
            else:
                previous = parent

        return previous

    def _get_title(self):
        """ Returns the title of the current section.
        """
        section = self.section
        if section is None:
            return ''

        return ('%s: %s' % (section.__class__.__name__, section.title))

    #-- Public Methods -------------------------------------------------------

    def init_tutor(self):
        """ Initials the tutor by creating the root section from the specified
            path.
        """
        path = self.path
        title = title_for(os.path.splitext(os.path.basename(path))[0])
        section = SectionFactory(title=title).trait_set(path=path).section
        if section is not None:
            self.section = self.root = section

#-------------------------------------------------------------------------
#  Run the program:
#-------------------------------------------------------------------------

# Only run the program if we were invoked from the command line:
if __name__ == '__main__':

    # Validate the command line arguments:
    if len(sys.argv) > 2:
        print(Usage)
        sys.exit(1)

    # Determine the root path to use for the tutorial files:
    if len(sys.argv) == 2:
        path = sys.argv[1]
    else:
        path = os.getcwd()

    # Create a tutor and display the tutorial:
    tutor = Tutor(home=os.path.dirname(sys.argv[0])).trait_set(
        path=path)
    if tutor.root is not None:
        tutor.configure_traits()
    else:
        print("""No traits tutorial found in %s.

Correct usage is: python tutor.py [tutorial_path]
where: tutorial_path = Path to the root of the traits tutorial.

If tutorial_path is omitted, the current directory is assumed to be the root of
the tutorial.""" % path)



"""
Implementation of an ImageEnumEditor demo plugin for the Traits UI demo program.

This demo shows each of the four styles of the ImageEnumEditor.
"""

from __future__ import absolute_import
from traits.api import HasTraits, Str, Trait
from traitsui.api import Item, Group, View, ImageEnumEditor

#-------------------------------------------------------------------------
#  Constants:
#-------------------------------------------------------------------------

# This list of image names (with the standard suffix "_origin") is used to
# construct an image enumeration trait to demonstrate the ImageEnumEditor.

image_list = ['top left', 'top right', 'bottom left', 'bottom right']


#-------------------------------------------------------------------------
#  Classes:
#-------------------------------------------------------------------------

class Dummy(HasTraits):
    """ Dummy class for ImageEnumEditor
    """
    x = Str
    view = View(" ")


class ImageEnumEditorDemo(HasTraits):
    """ This class specifies the details of the ImageEnumEditor demo.
    """

    # To demonstrate any given Trait editor, an appropriate Trait is required.
    image_from_list = Trait(editor=ImageEnumEditor(values=image_list,
                                                   prefix='@icons:',
                                                   suffix='_origin',
                                                   cols=4,
                                                   klass=Dummy),
                            *image_list)

    # Items are used to define the demo display - one Item per
    # editor style
    img_group = Group(Item('image_from_list', style='simple', label='Simple'),
                      Item('_'),
                      Item('image_from_list', style='custom', label='Custom'),
                      Item('_'),
                      Item('image_from_list', style='text', label='Text'),
                      Item('_'),
                      Item('image_from_list',
                           style='readonly',
                           label='ReadOnly'))

    #Demo view
    view1 = View(img_group,
                 title='ImageEnumEditor',
                 buttons=['OK'])


# Create the demo:
popup = ImageEnumEditorDemo()

# Run the demo (if invoked from the command line):
if __name__ == '__main__':
    popup.configure_traits()



"""
Implementation of a TupleEditor demo plugin for Traits UI demo program.

This demo shows each of the four styles of the TupleEditor.
"""

from __future__ import absolute_import
from traits.api import HasTraits, Tuple, Color, Range, Str
from traitsui.api import Item, Group, View

#-------------------------------------------------------------------------
#  Demo Class
#-------------------------------------------------------------------------


class TupleEditorDemo(HasTraits):
    """ This class specifies the details of the TupleEditor demo.
    """

    # To demonstrate any given Trait editor, an appropriate Trait is required.
    tuple = Tuple(Color, Range(1, 4), Str)

    # Display specification (one Item per editor style)
    tuple_group = Group(Item('tuple', style='simple', label='Simple'),
                        Item('_'),
                        Item('tuple', style='custom', label='Custom'),
                        Item('_'),
                        Item('tuple', style='text', label='Text'),
                        Item('_'),
                        Item('tuple', style='readonly', label='ReadOnly'))

    # Demo view
    view1 = View(tuple_group,
                 title='TupleEditor',
                 buttons=['OK'])


# Create the demo:
popup = TupleEditorDemo()

# Run the demo (if invoked from the command line):
if __name__ == '__main__':
    popup.configure_traits()



from __future__ import absolute_import
from traits.api import *

class EchoBox(HasTraits):
    input =  Str()
    output = Str()

    def _input_changed(self):
        self.output = self.input

EchoBox().configure_traits()




from __future__ import absolute_import
from pyface.image.image import *

from warnings import warn

warn(DeprecationWarning(
    "traitsui.image has been moved to pyface.image and will be removed in "
    "TraitsUI version 6.0"))



#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.

"""
Implementation of a SetEditor demo plugin for the Traits UI demo program.

The four tabs of this demo show variations on the interface as follows:

   Unord I:  Creates an alphabetized subset, has no "move all" options
   Unord II: Creates an alphabetized subset, has "move all" options
   Ord I:    Creates a set whose order is specified by the user, no "move all"
   Ord II:   Creates a set whose order is specifed by the user, has "move all"
"""

# Imports:
from __future__ import absolute_import
from traits.api \
    import HasTraits, List

from traitsui.api \
    import Item, Group, View, SetEditor

# Define the main demo class:


class SetEditorDemo(HasTraits):
    """ Defines the SetEditor demo class.
    """

    # Define a trait each for four SetEditor variants:
    unord_nma_set = List(editor=SetEditor(
        values=['kumquats', 'pomegranates', 'kiwi'],
        can_move_all=False,
        left_column_title='Available Fruit',
        right_column_title='Exotic Fruit Bowl'))

    unord_ma_set = List(editor=SetEditor(
        values=['kumquats', 'pomegranates', 'kiwi'],
        left_column_title='Available Fruit',
        right_column_title='Exotic Fruit Bowl'))

    ord_nma_set = List(editor=SetEditor(
        values=['apples', 'berries', 'cantaloupe'],
        ordered=True,
        can_move_all=False,
        left_column_title='Available Fruit',
        right_column_title='Fruit Bowl'))

    ord_ma_set = List(editor=SetEditor(
        values=['apples', 'berries', 'cantaloupe'],
        ordered=True,
        left_column_title='Available Fruit',
        right_column_title='Fruit Bowl'))

    # SetEditor display, unordered, no move-all buttons:
    no_nma_group = Group(
        Item('unord_nma_set', style='simple'),
        label='Unord I',
        show_labels=False
    )

    # SetEditor display, unordered, move-all buttons:
    no_ma_group = Group(
        Item('unord_ma_set', style='simple'),
        label='Unord II',
        show_labels=False
    )

    # SetEditor display, ordered, no move-all buttons:
    o_nma_group = Group(
        Item('ord_nma_set', style='simple'),
        label='Ord I',
        show_labels=False
    )

    # SetEditor display, ordered, move-all buttons:
    o_ma_group = Group(
        Item('ord_ma_set', style='simple'),
        label='Ord II',
        show_labels=False
    )

    # The view includes one group per data type. These will be displayed
    # on separate tabbed panels:
    view = View(
        no_nma_group,
        no_ma_group,
        o_nma_group,
        o_ma_group,
        title='SetEditor',
        buttons=['OK']
    )

# Create the demo:
demo = SetEditorDemo()

# Run the demo (if invoked from the command line):
if __name__ == '__main__':
    demo.configure_traits()



"""
Implementation of a DirectoryEditor demo plugin for Traits UI demo program.

This demo shows each of the four styles of the DirectoryEditor.
"""

from __future__ import absolute_import
from traits.api import HasTraits, Directory
from traitsui.api import Item, Group, View


#-------------------------------------------------------------------------
#  Demo Class
#-------------------------------------------------------------------------

class DirectoryEditorDemo(HasTraits):
    """ This class specifies the details of the DirectoryEditor demo.
    """

    # To demonstrate any given Trait editor, an appropriate Trait is required.
    dir_name = Directory

    # Display specification (one Item per editor style)
    dir_group = Group(Item('dir_name', style='simple', label='Simple'),
                      Item('_'),
                      Item('dir_name', style='custom', label='Custom'),
                      Item('_'),
                      Item('dir_name', style='text', label='Text'),
                      Item('_'),
                      Item('dir_name', style='readonly', label='ReadOnly'))

    # Demo view
    view1 = View(dir_group,
                 title='DirectoryEditor',
                 width=400,
                 buttons=['OK'])


# Create the demo:
popup = DirectoryEditorDemo()

# Run the demo (if invoked from the command line):
if __name__ == '__main__':
    popup.configure_traits()



from __future__ import absolute_import
import unittest

from traitsui.extras.demo import extract_docstring_from_source


class TestParseSource(unittest.TestCase):

    def test_simple_source(self):
        source_code = '\n'.join(['"""',
                                 'Docstring',
                                 '"""',
                                 'a = 1'])
        docstring, source = extract_docstring_from_source(source_code)
        self.assertEqual(docstring, '\nDocstring\n')
        self.assertEqual(source, 'a = 1')

    def test_alternate_quotes(self):
        source_code = '\n'.join(["'''",
                                 'Docstring',
                                 "'''",
                                 'a = 1'])
        docstring, source = extract_docstring_from_source(source_code)
        self.assertEqual(docstring, '\nDocstring\n')
        self.assertEqual(source, 'a = 1')

    def test_string_in_source(self):
        source_code = '\n'.join(['"""',
                                 'Docstring',
                                 '"""',
                                 '"string in source"',
                                 'a = 1'])
        docstring, source = extract_docstring_from_source(source_code)
        self.assertEqual(docstring, '\nDocstring\n')
        self.assertEqual(source, '\n'.join(['"string in source"',
                                            'a = 1']))

    def test_string_in_docstring(self):
        source_code = '\n'.join(['"""',
                                 'Docstring',
                                 '"string in docstring"',
                                 '"""',
                                 'a = 1'])
        docstring, source = extract_docstring_from_source(source_code)
        self.assertEqual(docstring, '\n'.join(['',
                                               'Docstring',
                                               '"string in docstring"',
                                               '']))
        self.assertEqual(source, '\n'.join(['a = 1']))

    def test_ignore_class_docstring(self):
        source_code = '\n'.join(['class Foo:',
                                 '    """Class docstring"""'])
        docstring, source = extract_docstring_from_source(source_code)
        self.assertEqual(docstring, '')
        self.assertEqual(source, '\n'.join(['class Foo:',
                                            '    """Class docstring"""']))

    def test_ignore_starting_comment(self):
        source_code = '\n'.join(['# Copyright notice.',
                                 '# Something about the author.',
                                 '"""',
                                 'Docstring',
                                 '"""',
                                 'a = 1'])
        docstring, source = extract_docstring_from_source(source_code)
        self.assertEqual(docstring, '\nDocstring\n')
        self.assertEqual(source, '\n'.join(['# Copyright notice.',
                                            '# Something about the author.',
                                            'a = 1']))

if __name__ == '__main__':
    unittest.main()



"""
Implementation of a ButtonEditor demo plugin for Traits UI demo program.

This demo shows each of the two styles of the ButtonEditor.
(As of this writing, they are identical.)
"""

from __future__ import absolute_import
from traits.api import HasTraits, Button
from traitsui.api import Item, View, Group
from traitsui.message import message


#-------------------------------------------------------------------------
#  Demo Class
#-------------------------------------------------------------------------

class ButtonEditorDemo(HasTraits):
    """ This class specifies the details of the ButtonEditor demo.
    """

    # To demonstrate any given Trait editor, an appropriate Trait is required.
    fire_event = Button('Click Me')

    def _fire_event_fired():
        message("Button clicked!")

    # ButtonEditor display
    # (Note that Text and ReadOnly versions are not applicable)
    event_group = Group(Item('fire_event', style='simple', label='Simple'),
                        Item('_'),
                        Item('fire_event', style='custom', label='Custom'),
                        Item('_'),
                        Item(label='[text style unavailable]'),
                        Item('_'),
                        Item(label='[read only style unavailable]'))

    # Demo view
    view1 = View(event_group,
                 title='ButtonEditor',
                 buttons=['OK'],
                 width=250)


# Create the demo:
popup = ButtonEditorDemo()

# Run the demo (if invoked from the command line):
if __name__ == '__main__':
    popup.configure_traits()



"""
Defining column-specific context menu in a Tabular Editor. Shows how the
example for the Table Editor can be adapted to work with a Tabular Editor.

The demo is a simple baseball scoring system, which lists each player and
their current batting statistics. After a given player has an at bat, you
right-click on the table cell corresponding to the player and the result of
the at-bat (e.g. 'S' = single) and select the 'Add' menu option to register
that the player hit a single and update the player's overall statistics.

This demo also illustrates the use of Property traits, and how using 'event'
meta-data can simplify event handling by collapsing an event that can
occur on a number of traits into a category of event, which can be handled by
a single event handler defined for the category (in this case, the category
is 'affects_average').
"""

from __future__ import absolute_import
from random import randint
from traits.api import HasStrictTraits, Str, Int, Float, List, Property
from traits.etsconfig.api import ETSConfig
from traitsui.api import View, Item, TabularEditor
from traitsui.menu import Menu, Action
from traitsui.tabular_adapter import TabularAdapter


# Define a custom tabular adapter for handling items which affect the player's
# batting average:
class PlayerAdapter(TabularAdapter):

    alignment = 'center'

    width = 0.09

    def get_menu(self, object, trait, row, column):
        column_name = self.column_map[column]
        if column_name not in ['name', 'average']:
            menu = Menu(Action(name='Add',
                               action='editor.adapter.add(item, column)'),
                        Action(name='Sub',
                               action='editor.adapter.sub(item, column)'))
            return menu
        else:
            return super(PlayerAdapter, self).get_menu(
                object, trait, row, column)

    def get_format(self, object, trait, row, column):
        column_name = self.column_map[column]
        if column_name == 'average':
            return '%0.3f'
        else:
            return super(PlayerAdapter, self).get_format(
                object, trait, row, column)

    def add(self, object, column):
        """ Increment the affected player statistic.
        """
        column_name = self.column_map[column]
        setattr(object, column_name, getattr(object, column_name) + 1)

    def sub(self, object, column):
        """ Decrement the affected player statistic.
        """
        column_name = self.column_map[column]
        setattr(object, column_name, getattr(object, column_name) - 1)


# The 'players' trait table editor:
columns = [
    ('Player Name', 'name'),
    ('AB', 'at_bats'),
    ('SO', 'strike_outs'),
    ('S', 'singles'),
    ('D', 'doubles'),
    ('T', 'triples'),
    ('HR', 'home_runs'),
    ('W', 'walks'),
    ('Ave', 'average'),
]

player_editor = TabularEditor(
    editable=True,
    auto_resize=True,
    auto_resize_rows=True,
    stretch_last_section=False,
    auto_update=True,
    adapter=PlayerAdapter(columns=columns),
    show_row_titles=ETSConfig.toolkit == 'qt4',
)


# 'Player' class:
class Player(HasStrictTraits):

    # Trait definitions:
    name = Str
    at_bats = Int
    strike_outs = Int(event='affects_average')
    singles = Int(event='affects_average')
    doubles = Int(event='affects_average')
    triples = Int(event='affects_average')
    home_runs = Int(event='affects_average')
    walks = Int
    average = Property(Float)

    def _get_average(self):
        """ Computes the player's batting average from the current statistics.
        """
        if self.at_bats == 0:
            return 0.0

        return float(self.singles + self.doubles +
                     self.triples + self.home_runs) / self.at_bats

    def _affects_average_changed(self):
        """ Handles an event that affects the player's batting average.
        """
        self.at_bats += 1


class Team(HasStrictTraits):

    # Trait definitions:
    players = List(Player)

    # Trait view definitions:
    traits_view = View(
        Item('players',
             show_label=False,
             editor=player_editor
             ),
        title='Baseball Scoring Demo',
        width=0.5,
        height=0.5,
        resizable=True
    )


def random_player(name):
    """ Generates and returns a random player.
    """
    p = Player(name=name,
               strike_outs=randint(0, 50),
               singles=randint(0, 50),
               doubles=randint(0, 20),
               triples=randint(0, 5),
               home_runs=randint(0, 30),
               walks=randint(0, 50))
    return p.trait_set(
        at_bats=p.strike_outs +
        p.singles +
        p.doubles +
        p.triples +
        p.home_runs +
        randint(
            100,
            200))

# Create the demo:
demo = view = Team(players=[random_player(name) for name in [
    'Dave', 'Mike', 'Joe', 'Tom', 'Dick', 'Harry', 'Dirk', 'Fields', 'Stretch'
]])

# Run the demo (if invoked from the command line):
if __name__ == '__main__':
    demo.configure_traits()



"""
Implementation of a CodeEditor demo plugin for Traits UI demo program.

This demo shows each of the four styles of the CodeEditor.
"""

from __future__ import absolute_import
from traits.api import HasTraits, Code
from traitsui.api import Item, Group, View

#-------------------------------------------------------------------------
#  Demo Class
#-------------------------------------------------------------------------


class CodeEditorDemo(HasTraits):
    """ This class specifies the details of the CodeEditor demo.
    """

    # To demonstrate any given Trait editor, an appropriate Trait is required.
    code_sample = Code('import sys\n\nsys.print("hello world!")')

    # Display specification
    code_group = Group(Item('code_sample', style='simple', label='Simple'),
                       Item('_'),
                       Item('code_sample', style='custom', label='Custom'),
                       Item('_'),
                       Item('code_sample', style='text', label='Text'),
                       Item('_'),
                       Item('code_sample', style='readonly', label='ReadOnly'))

    # Demo view
    view1 = View(code_group,
                 title='CodeEditor',
                 width=350,
                 buttons=['OK'])


# Create the demo:
popup = CodeEditorDemo()

# Run the demo (if invoked from the command line):
if __name__ == '__main__':
    popup.configure_traits()



#------------------------------------------------------------------------------
#
#  Copyright (c) 2005, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in enthought/LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#  Author: David C. Morrill
#  Date:   12/18/2004
#
#------------------------------------------------------------------------------

""" Defines the base class for the wxPython-based Traits UI modal and non-modal
    dialogs.
"""

from __future__ import absolute_import
import wx

from traits.api import HasPrivateTraits, Instance

from traitsui.base_panel import BasePanel as _BasePanel
from traitsui.menu import Action
from .editor import Editor


class ButtonEditor(Editor):
    """ Editor for buttons.
    """
    # Action associated with the button
    action = Instance(Action)

    def __init__(self, **traits):
        # XXX Why does this need to be an Editor subclass? -- CJW
        HasPrivateTraits.__init__(self, **traits)

    def perform(self, event):
        """ Handles the associated button being clicked.
        """
        handler = self.ui.handler
        self.ui.do_undoable(handler.perform, self.ui.info, self.action, event)


class BaseDialog(_BasePanel):
    """ Base class for Traits UI dialog boxes.
    """

    def default_icon(self):
        """ Return a default icon for a TraitsUI dialog. """
        from pyface.image_resource import ImageResource
        return ImageResource('frame.ico')

    def set_icon(self, icon=None):
        """ Sets the frame's icon.
        """
        from pyface.image_resource import ImageResource

        if not isinstance(icon, ImageResource):
            icon = self.default_icon()
        self.control.SetIcon(icon.create_icon())

    def add_statusbar(self):
        """ Adds a status bar to the dialog.
        """
        ui = self.ui
        statusbar = ui.view.statusbar
        context = ui.context
        if statusbar is not None:
            widths = []
            listeners = []
            control = wx.StatusBar(self.control)
            control.SetFieldsCount(len(statusbar))
            for i, item in enumerate(statusbar):
                width = abs(item.width)
                if width <= 1.0:
                    widths.append(-max(1, int(1000 * width)))
                else:
                    widths.append(int(width))

                set_text = self._set_status_text(control, i)
                name = item.name
                set_text(ui.get_extended_value(name))
                col = name.find('.')
                object = 'object'
                if col >= 0:
                    object = name[: col]
                    name = name[col + 1:]
                object = context[object]
                object.on_trait_change(set_text, name, dispatch='ui')
                listeners.append((object, set_text, name))

            control.SetStatusWidths(widths)
            self.control.SetStatusBar(control)
            ui._statusbar = listeners

    def _set_status_text(self, control, i):
        def set_status_text(text):
            control.SetStatusText(text, i)

        return set_status_text

    def add_menubar(self):
        """ Adds a menu bar to the dialog.
        """
        menubar = self.ui.view.menubar
        if menubar is not None:
            self._last_group = self._last_parent = None
            self.control.SetMenuBar(
                menubar.create_menu_bar(self.control, self))
            self._last_group = self._last_parent = None

    def add_toolbar(self):
        """ Adds a toolbar to the dialog.
        """
        toolbar = self.ui.view.toolbar
        if toolbar is not None:
            self._last_group = self._last_parent = None
            self.control.SetToolBar(
                toolbar.create_tool_bar(self.control, self))
            self._last_group = self._last_parent = None

    def add_button(self, action, sizer, method=None, enabled=True,
                   name=None, default=False):
        """ Creates a button.
        """
        ui = self.ui
        if ((action.defined_when != '') and
                (not ui.eval_when(action.defined_when))):
            return None

        if name is None:
            name = action.name
        id = action.id
        button = wx.Button(self.control, -1, name)
        button.Enable(enabled)
        if default:
            button.SetDefault()
        if (method is None) or (action.enabled_when != '') or (id != ''):
            editor = ButtonEditor(ui=ui,
                                  action=action,
                                  control=button)
            if id != '':
                ui.info.bind(id, editor)
            if action.visible_when != '':
                ui.add_visible(action.visible_when, editor)
            if action.enabled_when != '':
                ui.add_enabled(action.enabled_when, editor)
            if method is None:
                method = editor.perform
        wx.EVT_BUTTON(self.control, button.GetId(), method)
        sizer.Add(button, 0, wx.LEFT, 5)
        if action.tooltip != '':
            button.SetToolTipString(action.tooltip)
        return button



from unittest import TestCase

from traitsui.extras.demo import extract_docstring_from_source, parse_source


class TestDemo(TestCase):

    def test_extract_docstring_from_source(self):
        source_code = b""
        with self.assertRaises(TypeError):
            docstring, source = extract_docstring_from_source(source_code)

        source_code = u""
        docstring, source = extract_docstring_from_source(source_code)
        self.assertEqual((u"", u""), (docstring, source))

        source_code = u'''""" Module description """\nx=1\ny=2'''
        docstring, source = extract_docstring_from_source(source_code)
        expected = (u' Module description ', 'x=1\ny=2')
        self.assertEqual(expected, (docstring, source))

    def test_parse_source(self):
        docstring, source = parse_source('non-existent-file.<>|:')
        self.assertIn('Sorry, something went wrong.', docstring)



#------------------------------------------------------------------------------
#
#  Copyright (c) 2005, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in enthought/LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#  Author: David C. Morrill
#  Date:   12/13/2004
#
#------------------------------------------------------------------------------

""" Defines the tuple editor for the wxPython user interface toolkit.
"""

#-------------------------------------------------------------------------
#  Imports:
#-------------------------------------------------------------------------


# FIXME: ToolkitEditorFactory is a proxy class defined here just for backward
# compatibility. The class has been moved to the
# traitsui.editors.tuple_editor file.
from __future__ import absolute_import
from traitsui.editors.tuple_editor \
    import SimpleEditor as BaseSimpleEditor, ToolkitEditorFactory

from .editor \
    import Editor

#-------------------------------------------------------------------------
#  'SimpleEditor' class:
#-------------------------------------------------------------------------


class SimpleEditor(BaseSimpleEditor, Editor):
    """ Simple style of editor for tuples.

        The editor displays an editor for each of the fields in the tuple,
        based on the type of each field.
    """
    pass

### EOF #######################################################################



"""
Implemention of a ListEditor demo plugin for Traits UI demo program

This demo shows each of the four styles of ListEditor.
"""

from __future__ import absolute_import
from traits.api import HasTraits, List, Str
from traitsui.api import Item, Group, View

#-------------------------------------------------------------------------
#  Demo Class
#-------------------------------------------------------------------------


class ListEditorDemo(HasTraits):
    """ This class specifies the details of the BooleanEditor demo.
    """

    # The Trait to be displayed in the editor
    play_list = List(Str, ["The Merchant of Venice", "Hamlet", "MacBeth"])

    # Items are used to define display; one per editor style.
    list_group = Group(Item('play_list', style='simple', label='Simple'),
                       Item('_'),
                       Item('play_list', style='custom', label='Custom'),
                       Item('_'),
                       Item('play_list', style='text', label='Text'),
                       Item('_'),
                       Item('play_list',
                            style='readonly',
                            label='ReadOnly'))

    # Demo view
    view1 = View(list_group,
                 title='ListEditor',
                 buttons=['OK'],
                 height=400,
                 width=400)


# Create the demo:
popup = ListEditorDemo()

# Run the demo (if invoked from the command line):
if __name__ == '__main__':
    popup.configure_traits()



"""
Implementation of a CheckListEditor demo plugin for the Traits UI demo program.

For each of three CheckListEditor column formations, this demo shows
each of the four styles of the CheckListEditor.
"""

from __future__ import absolute_import
from traits.api import HasTraits, List
from traitsui.api import Item, Group, View, CheckListEditor

#-------------------------------------------------------------------------
#  Demo Class
#-------------------------------------------------------------------------


class CheckListEditorDemo(HasTraits):
    """ This class specifies the details of the CheckListEditor demo.
    """

    # Define a trait for each of three formations
    checklist_4col = List(editor=CheckListEditor(
        values=['one', 'two', 'three', 'four'],
        cols=4))

    checklist_2col = List(editor=CheckListEditor(
        values=['one', 'two', 'three', 'four'],
        cols=2))

    checklist_1col = List(editor=CheckListEditor(
        values=['one', 'two', 'three', 'four'],
        cols=1))

    # CheckListEditor display with four columns
    cl_4_group = Group(Item('checklist_4col', style='simple', label='Simple'),
                       Item('_'),
                       Item('checklist_4col', style='custom', label='Custom'),
                       Item('_'),
                       Item('checklist_4col', style='text', label='Text'),
                       Item('_'),
                       Item('checklist_4col', style='readonly', label='ReadOnly'),
                       label='4-column')

    # CheckListEditor display with two columns
    cl_2_group = Group(Item('checklist_2col', style='simple', label='Simple'),
                       Item('_'),
                       Item('checklist_2col', style='custom', label='Custom'),
                       Item('_'),
                       Item('checklist_2col', style='text', label='Text'),
                       Item('_'),
                       Item('checklist_2col', style='readonly', label='ReadOnly'),
                       label='2-column')

    # CheckListEditor display with one column
    cl_1_group = Group(Item('checklist_1col', style='simple', label='Simple'),
                       Item('_'),
                       Item('checklist_1col', style='custom', label='Custom'),
                       Item('_'),
                       Item('checklist_1col', style='text', label='Text'),
                       Item('_'),
                       Item('checklist_1col', style='readonly', label='ReadOnly'),
                       label='1-column')

    # The view includes one group per column formation.  These will be displayed
    # on separate tabbed panels.
    view1 = View(cl_4_group,
                 cl_2_group,
                 cl_1_group,
                 title='CheckListEditor',
                 buttons=['OK'])


# Create the demo:
popup = CheckListEditorDemo()

# Run the demo (if invoked from the command line):
if __name__ == '__main__':
    popup.configure_traits()



#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.


# configure_traits_view_buttons.py -- Sample code to demonstrate
#                                     configure_traits()

#--[Imports]--------------------------------------------------------------
from __future__ import absolute_import
from traits.api import HasTraits, Str, Int
from traitsui.api import View, Item
from traitsui.menu import OKButton, CancelButton

#--[Code]-----------------------------------------------------------------


class SimpleEmployee(HasTraits):
    first_name = Str
    last_name = Str
    department = Str

    employee_number = Str
    salary = Int

view1 = View(Item(name='first_name'),
             Item(name='last_name'),
             Item(name='department'),
             buttons=[OKButton, CancelButton])

sam = SimpleEmployee()
sam.configure_traits(view=view1)



#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.

# include_extra.py --- Example of Include object
#                      provided for subclasses
from __future__ import absolute_import
from traits.api import HasTraits, Int, Str
from traitsui.api import Group, Include, View


class Person(HasTraits):
    name = Str
    age = Int

    person_view = View('name', Include('extra'), 'age')


class LocatedPerson(Person):
    street = Str
    city = Str
    state = Str
    zip = Str

    extra = Group('street', 'city', 'state', 'zip')


Person().configure_traits()
LocatedPerson().configure_traits()



"""
Implementation of a CompoundEditor demo plugin for Traits UI demo program.

This demo shows each of the four styles of the CompoundEditor.
"""

from __future__ import absolute_import
from traits.api import HasTraits, Trait, Range
from traitsui.api import Item, Group, View

#-------------------------------------------------------------------------
#  Demo Class
#-------------------------------------------------------------------------


class CompoundEditorDemo(HasTraits):
    """ This class specifies the details of the CompoundEditor demo.
    """

    # To demonstrate any given Trait editor, an appropriate Trait is required.
    compound_trait = Trait(1, Range(1, 6), 'a', 'b', 'c', 'd', 'e', 'f')

    # Display specification (one Item per editor style)
    comp_group = Group(Item('compound_trait', style='simple', label='Simple'),
                       Item('_'),
                       Item('compound_trait', style='custom', label='Custom'),
                       Item('_'),
                       Item('compound_trait', style='text', label='Text'),
                       Item('_'),
                       Item('compound_trait',
                            style='readonly',
                            label='ReadOnly'))

    # Demo view
    view1 = View(comp_group,
                 title='CompoundEditor',
                 buttons=['OK'])


# Create the demo:
popup = CompoundEditorDemo()

# Run the demo (if invoked from the command line):
if __name__ == '__main__':
    popup.configure_traits()



#------------------------------------------------------------------------------
# Copyright (c) 2007, Riverbank Computing Limited
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD license.
# However, when used with the GPL version of PyQt the additional terms
# described in the PyQt GPL exception also apply

#
# Author: Riverbank Computing Limited
#------------------------------------------------------------------------------

""" Converts a QKeyEvent to a standardized "name".
"""

#-------------------------------------------------------------------------
#  Imports:
#-------------------------------------------------------------------------

from __future__ import absolute_import
from pyface.qt import QtCore

#-------------------------------------------------------------------------
#  Constants:
#-------------------------------------------------------------------------

# Mapping from PyQt keypad key names to Pyface key names.
keypad_map = {
    QtCore.Qt.Key_Enter: 'Enter',
    QtCore.Qt.Key_0: 'Numpad 0',
    QtCore.Qt.Key_1: 'Numpad 1',
    QtCore.Qt.Key_2: 'Numpad 2',
    QtCore.Qt.Key_3: 'Numpad 3',
    QtCore.Qt.Key_4: 'Numpad 4',
    QtCore.Qt.Key_5: 'Numpad 5',
    QtCore.Qt.Key_6: 'Numpad 6',
    QtCore.Qt.Key_7: 'Numpad 7',
    QtCore.Qt.Key_8: 'Numpad 8',
    QtCore.Qt.Key_9: 'Numpad 9',
    QtCore.Qt.Key_Asterisk: 'Multiply',
    QtCore.Qt.Key_Plus: 'Add',
    QtCore.Qt.Key_Comma: 'Separator',
    QtCore.Qt.Key_Minus: 'Subtract',
    QtCore.Qt.Key_Period: 'Decimal',
    QtCore.Qt.Key_Slash: 'Divide'
}

# Mapping from PyQt non-keypad key names to Pyface key names.
key_map = {
    QtCore.Qt.Key_0: '0',
    QtCore.Qt.Key_1: '1',
    QtCore.Qt.Key_2: '2',
    QtCore.Qt.Key_3: '3',
    QtCore.Qt.Key_4: '4',
    QtCore.Qt.Key_5: '5',
    QtCore.Qt.Key_6: '6',
    QtCore.Qt.Key_7: '7',
    QtCore.Qt.Key_8: '8',
    QtCore.Qt.Key_9: '9',
    QtCore.Qt.Key_A: 'A',
    QtCore.Qt.Key_B: 'B',
    QtCore.Qt.Key_C: 'C',
    QtCore.Qt.Key_D: 'D',
    QtCore.Qt.Key_E: 'E',
    QtCore.Qt.Key_F: 'F',
    QtCore.Qt.Key_G: 'G',
    QtCore.Qt.Key_H: 'H',
    QtCore.Qt.Key_I: 'I',
    QtCore.Qt.Key_J: 'J',
    QtCore.Qt.Key_K: 'K',
    QtCore.Qt.Key_L: 'L',
    QtCore.Qt.Key_M: 'M',
    QtCore.Qt.Key_N: 'N',
    QtCore.Qt.Key_O: 'O',
    QtCore.Qt.Key_P: 'P',
    QtCore.Qt.Key_Q: 'Q',
    QtCore.Qt.Key_R: 'R',
    QtCore.Qt.Key_S: 'S',
    QtCore.Qt.Key_T: 'T',
    QtCore.Qt.Key_U: 'U',
    QtCore.Qt.Key_V: 'V',
    QtCore.Qt.Key_W: 'W',
    QtCore.Qt.Key_X: 'X',
    QtCore.Qt.Key_Y: 'Y',
    QtCore.Qt.Key_Z: 'Z',
    QtCore.Qt.Key_Space: 'Space',
    QtCore.Qt.Key_Backspace: 'Backspace',
    QtCore.Qt.Key_Tab: 'Tab',
    QtCore.Qt.Key_Enter: 'Enter',
    QtCore.Qt.Key_Return: 'Return',
    QtCore.Qt.Key_Escape: 'Esc',
    QtCore.Qt.Key_Delete: 'Delete',
    QtCore.Qt.Key_Cancel: 'Cancel',
    QtCore.Qt.Key_Clear: 'Clear',
    QtCore.Qt.Key_Shift: 'Shift',
    QtCore.Qt.Key_Menu: 'Menu',
    QtCore.Qt.Key_Pause: 'Pause',
    QtCore.Qt.Key_PageUp: 'Page Up',
    QtCore.Qt.Key_PageDown: 'Page Down',
    QtCore.Qt.Key_End: 'End',
    QtCore.Qt.Key_Home: 'Home',
    QtCore.Qt.Key_Left: 'Left',
    QtCore.Qt.Key_Up: 'Up',
    QtCore.Qt.Key_Right: 'Right',
    QtCore.Qt.Key_Down: 'Down',
    QtCore.Qt.Key_Select: 'Select',
    QtCore.Qt.Key_Print: 'Print',
    QtCore.Qt.Key_Execute: 'Execute',
    QtCore.Qt.Key_Insert: 'Insert',
    QtCore.Qt.Key_Help: 'Help',
    QtCore.Qt.Key_F1: 'F1',
    QtCore.Qt.Key_F2: 'F2',
    QtCore.Qt.Key_F3: 'F3',
    QtCore.Qt.Key_F4: 'F4',
    QtCore.Qt.Key_F5: 'F5',
    QtCore.Qt.Key_F6: 'F6',
    QtCore.Qt.Key_F7: 'F7',
    QtCore.Qt.Key_F8: 'F8',
    QtCore.Qt.Key_F9: 'F9',
    QtCore.Qt.Key_F10: 'F10',
    QtCore.Qt.Key_F11: 'F11',
    QtCore.Qt.Key_F12: 'F12',
    QtCore.Qt.Key_F13: 'F13',
    QtCore.Qt.Key_F14: 'F14',
    QtCore.Qt.Key_F15: 'F15',
    QtCore.Qt.Key_F16: 'F16',
    QtCore.Qt.Key_F17: 'F17',
    QtCore.Qt.Key_F18: 'F18',
    QtCore.Qt.Key_F19: 'F19',
    QtCore.Qt.Key_F20: 'F20',
    QtCore.Qt.Key_F21: 'F21',
    QtCore.Qt.Key_F22: 'F22',
    QtCore.Qt.Key_F23: 'F23',
    QtCore.Qt.Key_F24: 'F24',
    QtCore.Qt.Key_NumLock: 'Num Lock',
    QtCore.Qt.Key_ScrollLock: 'Scroll Lock'
}

#-------------------------------------------------------------------------
#  Converts a keystroke event into a corresponding key name:
#-------------------------------------------------------------------------


def key_event_to_name(event):
    """ Converts a keystroke event into a corresponding key name.
    """
    key_code = event.key()
    modifiers = event.modifiers()
    if modifiers & QtCore.Qt.KeypadModifier:
        key = keypad_map.get(key_code)
    else:
        key = None
    if key is None:
        key = key_map.get(key_code, 'Unknown-Key')

    name = ''
    if modifiers & QtCore.Qt.ControlModifier:
        name += 'Ctrl'

    if modifiers & QtCore.Qt.AltModifier:
        name += '-Alt' if name else 'Alt'

    if modifiers & QtCore.Qt.MetaModifier:
        name += '-Meta' if name else 'Meta'

    if modifiers & QtCore.Qt.ShiftModifier and (
            (name != '') or (len(key) > 1)):
        name += '-Shift' if name else 'Shift'

    if key:
        if name:
            name += '-'
        name += key
    return name



#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.

"""
This demo illustrates use of the LEDEditor for displaying numeric values
using a simulated LED display control.
"""

from __future__ import absolute_import
from threading \
    import Thread

from time \
    import sleep

from traits.api \
    import HasTraits, Instance, Int, Bool, Float

from traitsui.api \
    import View, Item, HGroup, Handler, UIInfo, spring

from traitsui.wx.extra.led_editor \
    import LEDEditor

# Handler class for the LEDDemo class view:


class LEDDemoHandler(Handler):

    # The UIInfo object associated with the UI:
    info = Instance(UIInfo)

    # Is the demo currently running:
    running = Bool(True)

    # Is the thread still alive?
    alive = Bool(True)

    def init(self, info):
        self.info = info
        Thread(target=self._update_counter).start()

    def closed(self, info, is_ok):
        self.running = False
        while self.alive:
            sleep(.05)

    def _update_counter(self):
        while self.running:
            self.info.object.counter1 += 1
            self.info.object.counter2 += .001
            sleep(.01)
        self.alive = False

# The main demo class:


class LEDDemo(HasTraits):

    # A counter to display:
    counter1 = Int

    # A floating point value to display:
    counter2 = Float

    # The traits view:
    view = View(
        Item('counter1',
             label='Left aligned',
             editor=LEDEditor(alignment='left')
             ),
        Item('counter1',
             label='Center aligned',
             editor=LEDEditor(alignment='center')
             ),
        Item('counter1',
             label='Right aligned',
             editor=LEDEditor()  # default = 'right' aligned
             ),
        Item('counter2',
             label='Float value',
             editor=LEDEditor(format_str='%.3f')
             ),
        '_',
        HGroup(
            Item('counter1',
                 label='Left',
                 height=-40,
                 width=120,
                 editor=LEDEditor(alignment='left')
                 ),
            spring,
            Item('counter1',
                 label='Center',
                 height=-40,
                 width=120,
                 editor=LEDEditor(alignment='center')
                 ),
            spring,
            Item('counter1',
                 label='Right',
                 height=-40,
                 width=120,
                 editor=LEDEditor()  # default = 'right' aligned
                 ),
            spring,
            Item('counter2',
                 label='Float',
                 height=-40,
                 width=120,
                 editor=LEDEditor(format_str='%.3f')
                 )
        ),
        title='LED Editor Demo',
        buttons=['OK'],
        handler=LEDDemoHandler
    )

# Create the demo:
demo = LEDDemo()

# Run the demo (if invoked from the command line):
if __name__ == '__main__':
    demo.configure_traits()
